@@ -8,12 +8,13 @@ from __future__ import annotations
 import os
 import sys
 from emlib.common import runonce
+from functools import cache
 import logging
 
 
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import List, Tuple, Sequence, Optional
+    from typing import Sequence
 
 _DEFAULT_FONT = ("Helvetica", 11)
 
@@ -37,7 +38,7 @@ filters = {
 
 
 @runonce
-def _has_qt() -> bool:
+def _has_qt5() -> bool:
     try:
         from PyQt5 import QtWidgets
         return True
@@ -45,8 +46,20 @@ def _has_qt() -> bool:
         return False
 
 
+@runonce
+def _has_qt6() -> bool:
+    try:
+        from PyQt6 import QtWidgets
+        return True
+    except ImportError:
+        return False
+
+
 @runonce
 def _has_tk() -> bool:
+    # tk in linux with python 3.13 is often broken
+    if sys.platform == 'linux' and sys.version_info.minor >= 13:
+        return False
     try:
         import tkinter
         return True
@@ -54,17 +67,22 @@ def _has_tk() -> bool:
         return False
 
 
-def _resolveBackend(backend=''):
-    if sys.platform == 'darwin':
-        backend = 'qt'
-    elif not backend:
-        if _has_qt():
-            backend = 'qt'
+@cache
+def _resolveBackend(backend='') -> str:
+    if not backend:
+        if _has_qt6():
+            backend = 'qt6'
+        elif _has_qt5():
+            backend = 'qt5'
         elif _has_tk():
             backend = 'tk'
         else:
-            raise RuntimeError("No backends available. Install pyqt5 via 'pip install pyqt5'")
-    if backend == 'qt' and not _has_qt():
+            raise RuntimeError("No backends available. Install pyqt6 via 'pip install pyqt6'")
+
+    if backend == 'qt6' and not _has_qt6():
+        raise RuntimeError("pyqt6 is needed, but is not installed. Install it via 'pip install pyqt6'")
+
+    if backend == 'qt5' and not _has_qt5():
         raise RuntimeError("pyqt5 is needed, but is not installed. Install it via 'pip install pyqt5'")
     return backend
 
@@ -81,10 +99,14 @@ def showInfo(msg: str, title: str = "Info", font=None, icon='', backend=''
         icon: either None or one of 'question', 'information', 'warning', 'critical'
     """
     backend = _resolveBackend(backend)
-    if backend == 'qt':
+    if backend == 'qt5':
         from . import _dialogsqt
         return _dialogsqt.showInfo(msg=msg, title=title, font=font, icon=icon)
 
+    elif backend == 'qt6':
+        from . import _dialogsqt6
+        return _dialogsqt6.showInfo(msg=msg, title=title, font=font, icon=icon)
+
     from tkinter import ttk, Tk
     try:
         from ttkthemes import ThemedTk
@@ -125,10 +147,17 @@ def selectFile(directory='', filter="All (*.*)", title="Open file",
     """
     backend = _resolveBackend(backend)
 
-    if backend == 'qt':
+    if backend == 'qt5':
         from . import _dialogsqt
         return _dialogsqt.selectFile(directory=directory, filter=filter, title=title)
 
+    elif backend == 'qt6':
+        from . import _dialogsqt6
+        return _dialogsqt6.selectFile(directory=directory, filter=filter, title=title)
+
+    if not _has_tk():
+        raise ValueError("tk is not available")
+
     from tkinter import ttk, Tk
     try:
         from ttkthemes import ThemedTk
@@ -149,7 +178,7 @@ def selectFile(directory='', filter="All (*.*)", title="Open file",
     return path
 
 
-def _tkParseFilter(filter: str) -> list[Tuple[str, str]]:
+def _tkParseFilter(filter: str) -> list[tuple[str, str]]:
     # A filter has the form <name1> (<wildcard1>, <wildcard2>, ...);; name2...
     parts = filter.split(";;")
     out = []
@@ -167,18 +196,7 @@ def _tkParseFilter(filter: str) -> list[Tuple[str, str]]:
     return out
 
 
-@runonce
-def _tkOk() -> bool:
-    try:
-        from ttkthemes import ThemedTk
-        from tkinter import filedialog
-        root = ThemedTk(theme='breeze')
-        return True
-    except:
-        return False
-
-
-def _saveDialogTk(filter="All (*.*)", title="Save file", directory: str = "~") -> str:
+def _saveDialogTk(filter="All (*.*)", title="Save file", directory="~") -> str:
     from tkinter import ttk, Tk
     try:
         from ttkthemes import ThemedTk
@@ -195,7 +213,7 @@ def _saveDialogTk(filter="All (*.*)", title="Save file", directory: str = "~") -
     return path
 
 
-def saveDialog(filter="All (*.*)", title="Save file", directory: str = "~", backend=''
+def saveDialog(filter="All (*.*)", title="Save file", directory="~", backend=''
                ) -> str:
     """
     Open a dialog to save a file.
@@ -219,9 +237,13 @@ def saveDialog(filter="All (*.*)", title="Save file", directory: str = "~", back
     if not directory:
         directory = "~"
     directory = os.path.expanduser(directory)
-    if backend == 'qt':
+    if backend == 'qt5':
         from . import _dialogsqt
         return _dialogsqt.saveDialog(filter=filter, title=title, directory=directory)
+    if backend == 'qt6':
+        from . import _dialogsqt6
+        return _dialogsqt6.saveDialog(filter=filter, title=title, directory=directory)
+
     else:
         if sys.platform == 'darwin':
             raise RuntimeError("tk backend not supported in macos")
@@ -291,7 +313,7 @@ def selectItems(items: Sequence[str],
         caseSensitive: if True, filtering is case sensitive
         ensureSelection: if True, raises a ValueError exception is no selection
             was done
-        backend: if given, one of 'qt', 'tk'
+        backend: if given, one of 'qt6', 'qt5', 'tk'
 
     Returns:
         a list of selected items, or an empty list if the user aborted
@@ -303,16 +325,24 @@ def selectItems(items: Sequence[str],
                                  listFont=listFont, scrollbar=scrollbar, width=width,
                                  numlines=numlines, caseSensitive=caseSensitive,
                                  ensureSelection=ensureSelection)
-    elif backend == 'qt':
-        if not _has_qt():
-            raise RuntimeError("pyqt5 not installed. Install it via 'pip install pyqt5'")
+    elif backend == 'qt6':
+        if not _has_qt6():
+            raise RuntimeError("qt6 not installed. Install it via 'pip install pyqt6'")
+        logger.info("Multiple item selection is not supported in qt at the moment")
+        from . import _dialogsqt6
+        out = _dialogsqt6.selectItem(items=items, title=title, listFont=listFont,
+                                    entryFont=entryFont)
+        return [out] if out is not None else []
+    elif backend == 'qt5':
+        if not _has_qt5():
+            raise RuntimeError("qt5 not installed. Install it via 'pip install pyqt5'")
         logger.info("Multiple item selection is not supported in qt at the moment")
         from . import _dialogsqt
         out = _dialogsqt.selectItem(items=items, title=title, listFont=listFont,
                                     entryFont=entryFont)
         return [out] if out is not None else []
     else:
-        raise ValueError("Backends supported: 'qt', 'tk'")
+        raise ValueError("Backends supported: 'qt5', 'qt6', 'tk'")
 
 
 def _selectFromListTk(items: Sequence[str], title="Select", entryFont=('Arial', 15),
@@ -343,11 +373,12 @@ def _selectFromListTk(items: Sequence[str], title="Select", entryFont=('Arial',
         logger.error("macOS is not supported")
 
     from tkinter import ttk, Tk
-    try:
-        from ttkthemes import ThemedTk
-        root = ThemedTk(theme="breeze")
-    except ImportError:
-        root = Tk()
+    root = Tk()
+    #try:
+    #    from ttkthemes import ThemedTk
+    #    root = ThemedTk(theme="breeze")
+    #except ImportError:
+    #    root = Tk()
 
     import tkinter as tk
     from tkinter.font import Font
