@@ -80,15 +80,15 @@ import inspect
 import dataclasses
 import subprocess
 import tempfile
-from typing import Any, List, Optional as Opt, Callable, Set, Type, Dict, Tuple
+from typing import Any, Callable
 import logging
 from emlib import iterlib, textlib
-import io
+from functools import cache
 import enum
 import re
 
 
-logger = logging.getLogger("emlib.doctools")
+logger = logging.getLogger("doctools")
 
 
 @dataclasses.dataclass
@@ -136,8 +136,8 @@ class ParsedDef:
     """
     name: str = ''
     kind: ObjectKind = ObjectKind.Unknown
-    params: List[Param] = dataclasses.field(default_factory=list)
-    returns: Opt[Param] = None
+    params: list[Param] = dataclasses.field(default_factory=list)
+    returns: Param | None = None
     shortDescr: str = ''
     longDescr: str = ''
     embeddedSignature: str = ''
@@ -151,6 +151,9 @@ class ParsedDef:
         if self.params is None:
             self.params = []
 
+        if not self.shortDescr and self.longDescr:
+            self.shortDescr = textlib.firstSentence(self.longDescr)
+
     def prefix(self) -> str:
         if self.kind == ObjectKind.Class:
             return "class"
@@ -207,6 +210,9 @@ class RenderConfig:
     includeInheritedMethods: bool = False
     """If True, add inherited methods to the documentation of each class"""
 
+    generateClassTOC: bool = True
+    """If True, add a TOC to a class definition"""
+
 
 class ParseError(Exception):
     pass
@@ -228,19 +234,28 @@ def _parseProperty(p) -> ParsedDef:
                      shortDescr=docs)
 
 
-_mdreplacer = textlib.makeReplacer({'_': '\_',
-                                    '*': '\*'})
+_mdreplacer = textlib.makeReplacer({'_': r'\_',
+                                    '*': r'\*'})
 
 
 def markdownEscape(s: str) -> str:
     """
     Escape s as markdown
 
-    Example
-    -------
+    Args:
+        s: the string to escape
+
+    Returns:
+        the escaped test
+
+
+    **Example**
+
+
+    .. code::
 
         >>> markdownEscape("my_title_with_underscore")
-        my\_title\_with\_underscore
+        my\\_title\\_with\\_underscore
     
     """
     return _mdreplacer(s)
@@ -294,11 +309,13 @@ def parseDef(obj) -> ParsedDef:
                     embeddedSignature, docstr = docstr.split("\n", maxsplit=1)
             except ValueError:
                 pass
+    elif inspect.isclass(obj):
+        kind = ObjectKind.Class
     else:
         raise ParseError(f"Could not parse {obj}")
 
     objName = obj.__qualname__
-    sig: Opt[inspect.Signature]
+    sig: inspect.Signature | None
     try:
         sig = inspect.signature(obj)
     except ValueError:
@@ -314,7 +331,7 @@ def parseDef(obj) -> ParsedDef:
     if sig and any(p for p in sig.parameters if p != 'self') and not docstring.params:
         logger.warning(f"{objName}: No parameters declared in the docstring")
 
-    paramNameToParam: Dict[str, Param] = {p.name: p for p in docstring.params}
+    paramNameToParam: dict[str, Param] = {p.name: p for p in docstring.params}
     shortDescr = docstring.shortDescr
     longDescr = docstring.longDescr
     params = []
@@ -347,7 +364,7 @@ def parseDef(obj) -> ParsedDef:
         for p in docstring.params:
             params.append(p)
 
-    returns: Opt[Param]
+    returns: Param | None
     if not docstring.returns:
         if sig and sig.return_annotation != inspect._empty:
             returns = Param(name='returns', type=sig.return_annotation)
@@ -400,8 +417,8 @@ def parseDocstring(docstr: str, fmt='google') -> ParsedDef:
         lines = lines[1:]
     descrLines = []
     context = None
-    params: List[Param] = []
-    returnLines: List[str] = []
+    params: list[Param] = []
+    returnLines: list[str] = []
     for line in lines:
         linestrip = line.strip()
         if context and not linestrip:
@@ -437,7 +454,7 @@ def parseDocstring(docstr: str, fmt='google') -> ParsedDef:
             returnLines.append(linestrip)
 
     longDescr = "\n".join(descrLines)
-    returns: Opt[Param]
+    returns: Param | None
     returnLines = [l for l in returnLines
                    if l.strip()]
     if returnLines:
@@ -492,13 +509,13 @@ def formatSignature(name: str, signature: str, maxwidth: int=70,
 
     Transforms a signature like::
 
-        '(a: int, b: List[foo], c=200, ..., z=None) -> List[foo]'
+        '(a: int, b: list[foo], c=200, ..., z=None) -> list[foo]'
 
     into::
 
-        def func(a: int, b:List[foo], c=200, signa, b=200, ...
+        def func(a: int, b:list[foo], c=200, signa, b=200, ...
                  z=None
-                 ) -> List[foo]:
+                 ) -> list[foo]:
 
     Args:
         name: the name of the function
@@ -557,7 +574,7 @@ def formatSignature(name: str, signature: str, maxwidth: int=70,
     return "\n".join(lines)
 
 
-def _mdParam(param:Param, maxwidth=70, indent=4) -> List[str]:
+def _mdParam(param:Param, maxwidth=70, indent=4) -> list[str]:
     s = f"* **{param.name}**"
     if param.type:
         s += f" (`{param.type}`)"
@@ -572,7 +589,7 @@ def _rstConvertNotesToMarkdown(rst: str) -> str:
     return rst
 
 
-def _rstToMarkdown(rst: str, startLevel=1) -> str:
+def _rstToMarkdown(rst: str, indentLevel=1) -> str:
     rst = _rstConvertNotesToMarkdown(rst)
     rstfile = tempfile.mktemp(suffix=".rst")
     mdfile = os.path.splitext(rstfile)[0] + ".md"
@@ -580,11 +597,11 @@ def _rstToMarkdown(rst: str, startLevel=1) -> str:
     proc = subprocess.Popen(["pandoc", "-f", "rst", "-t", "markdown", rstfile], stdout=subprocess.PIPE)
     proc.wait()
     mdstr = proc.stdout.read().decode("utf-8")
-    mdstr = markdownReplaceHeadings(mdstr, startLevel=startLevel)
+    mdstr = markdownReplaceHeadings(mdstr, indentLevel=indentLevel)
     return mdstr
 
 
-def markdownReplaceHeadings(s: str, startLevel=1, normalize=True) -> str:
+def markdownReplaceHeadings(s: str, indentLevel=1, normalize=True) -> str:
     """
     Replaces any heading of the form::
 
@@ -593,15 +610,15 @@ def markdownReplaceHeadings(s: str, startLevel=1, normalize=True) -> str:
 
     Args:
         s: the markdown text
-        startLevel: the heading start level
+        indentLevel: the heading start level
         normalize: if True, the highest heading in s will be forced to become
-            a `startLevel` heading.
+            a `indentLevel` heading.
 
     Returns:
         the modified markdown text
     """
     lines = s.splitlines()
-    out: List[str] = []
+    out: list[str] = []
     roothnum = 100
     skip = False
     lines.append("")
@@ -628,9 +645,9 @@ def markdownReplaceHeadings(s: str, startLevel=1, normalize=True) -> str:
             skip = True
         else:
             out.append(line)
-    if startLevel == 1 and not normalize:
+    if indentLevel == 1 and not normalize:
         return "\n".join(out)
-    # hnum     startLevel   roothnum   hnumnow
+    # hnum     indentLevel   roothnum   hnumnow
     #    1              1          1         1
     #    1              2          1         2
     #    2              1          1         2
@@ -648,7 +665,7 @@ def markdownReplaceHeadings(s: str, startLevel=1, normalize=True) -> str:
         elif line.startswith("#"):
             hstr, text = line.split(maxsplit=1)
             hnum = len(hstr)
-            hnumnow = hnum - roothnum + startLevel
+            hnumnow = hnum - roothnum + indentLevel
             if hnumnow != hnum:
                 out2.append(markdownHeader(text, hnumnow))
             else:
@@ -666,7 +683,7 @@ def _guessDocFormat(docstring: str) -> str:
             formats.add("markdown")
         if linestrip.startswith(".. ") or linestrip.endswith("::"):
             formats.add("rst")
-        if re.search("``\w+``", linestrip) or ":meth:" in linestrip or ":class:" in linestrip:
+        if re.search(r"``\w+``", linestrip) or ":meth:" in linestrip or ":class:" in linestrip:
             formats.add("rst")
         if "~~" in linestrip:
             formats.add("rst")
@@ -677,11 +694,11 @@ def _guessDocFormat(docstring: str) -> str:
     return formats.pop()
 
 
-def _renderAttributeMarkdown(parsed: ParsedDef, startLevel=0):
-    if startLevel == 0:
+def _renderAttributeMarkdown(parsed: ParsedDef, indentLevel=0):
+    if indentLevel == 0:
         s = f"* **{parsed.name}**"
     else:
-        s = markdownHeader(parsed.name, startLevel)
+        s = markdownHeader(parsed.name, indentLevel)
 
     descr = parsed.shortDescr
     if descr:
@@ -691,11 +708,11 @@ def _renderAttributeMarkdown(parsed: ParsedDef, startLevel=0):
     return s
 
 
-def _renderDocMarkdown(parsed: ParsedDef, startLevel:int, renderConfig: RenderConfig
+def _renderDocMarkdown(parsed: ParsedDef, indentLevel:int, renderConfig: RenderConfig
                        ) -> str:
     if parsed.kind == ObjectKind.Attribute:
-        level = startLevel if renderConfig.attributesAreHeadings else 0
-        return _renderAttributeMarkdown(parsed, startLevel=level)
+        level = indentLevel if renderConfig.attributesAreHeadings else 0
+        return _renderAttributeMarkdown(parsed, indentLevel=level)
     arglines = []
 
     params = [p for p in parsed.params if p.name != 'self'] # and (p.descr or p.default or p.type)]
@@ -724,14 +741,14 @@ def _renderDocMarkdown(parsed: ParsedDef, startLevel:int, renderConfig: RenderCo
         if docfmt == 'auto':
             docfmt = _guessDocFormat(parsed.longDescr)
         if docfmt == 'rst':
-            longdescr = _rstToMarkdown(parsed.longDescr, startLevel=startLevel+1)
+            longdescr = _rstToMarkdown(parsed.longDescr, indentLevel=indentLevel+1)
         elif docfmt == 'markdown':
             longdescr = parsed.longDescr
         else:
             raise ValueError(f"doc format {docfmt} not supported")
-    longdescr = markdownReplaceHeadings(longdescr, startLevel=startLevel+1, normalize=True)
+    longdescr = markdownReplaceHeadings(longdescr, indentLevel=indentLevel+1, normalize=True)
     componentName = parsed.name if not renderConfig.splitName else parsed.name.split(".")[-1]
-    blocks = [markdownHeader(componentName, headernum=startLevel)]
+    blocks = [markdownHeader(componentName, headernum=indentLevel)]
     if parsed.embeddedSignature:
         fmtsig = parsed.embeddedSignature
     else:
@@ -756,21 +773,21 @@ def _renderDocMarkdown(parsed: ParsedDef, startLevel:int, renderConfig: RenderCo
     return textwrap.dedent(s)
 
 
-def renderDocumentation(parsed: ParsedDef, renderConfig: RenderConfig, startLevel:int
+def renderDocumentation(parsed: ParsedDef, renderConfig: RenderConfig, indentLevel:int
                         ) -> str:
     """
     Renders the parsed function / method /  property as documentation in the given format
 
     Args:
         parsed: the result of calling parseDef on a function or method
-        startLevel: the heading level to use as root level for the documentation
+        indentLevel: the heading level to use as root level for the documentation
         renderConfig: a RenderConfig
 
     Returns:
         the generated documentation as string
     """
     if renderConfig.fmt == 'markdown':
-        return _renderDocMarkdown(parsed, startLevel=startLevel, renderConfig=renderConfig)
+        return _renderDocMarkdown(parsed, indentLevel=indentLevel, renderConfig=renderConfig)
     else:
         raise ValueError(f"format {renderConfig.fmt} not supported)")
 
@@ -795,8 +812,8 @@ def fullname(obj, includeModule=True) -> str:
     return module+'.'+cls.__qualname__ if includeModule else cls.__qualname__
 
 
-def generateDocsForFunctions(funcs: List[Callable], renderConfig: RenderConfig=None,
-                             title:str=None, pretext:str = None, startLevel=1
+def generateDocsForFunctions(funcs: list[Callable], renderConfig: RenderConfig=None,
+                             title:str=None, pretext:str = None, indentLevel=1
                              ) -> str:
     """
     Collects documentation for multiple functions in one string
@@ -805,7 +822,7 @@ def generateDocsForFunctions(funcs: List[Callable], renderConfig: RenderConfig=N
         funcs: the funcs to parse
         title: a title to use before the generated code
         pretext: a text between the title and the generated code
-        startLevel: the heading start level
+        indentLevel: the heading start level
         renderConfig: a RenderConfig
 
     Returns:
@@ -816,11 +833,11 @@ def generateDocsForFunctions(funcs: List[Callable], renderConfig: RenderConfig=N
     """
     if renderConfig is None:
         renderConfig = RenderConfig()
-    lines: List[str] = []
+    lines: list[str] = []
     sep = "\n----------\n"
     _ = lines.append
     if title:
-        _(markdownHeader(title, startLevel))
+        _(markdownHeader(title, indentLevel))
         _("")
 
     if pretext:
@@ -828,14 +845,14 @@ def generateDocsForFunctions(funcs: List[Callable], renderConfig: RenderConfig=N
         _(sep)
 
     lasti = len(funcs)-1
-    startLevelForFuncs = startLevel
+    indentLevelForFuncs = indentLevel
     for i, func in enumerate(funcs):
         try:
             parsed = parseDef(func)
         except ParseError:
             logger.error(f"Could not parse object {func}")
             continue
-        docstr = renderDocumentation(parsed, startLevel=startLevelForFuncs,
+        docstr = renderDocumentation(parsed, indentLevel=indentLevelForFuncs,
                                      renderConfig=renderConfig)
         _(docstr)
         if i < lasti:
@@ -850,12 +867,17 @@ class ClassMembers:
 
     A member can be an attribute (a `@property`) or a method
     """
-    properties: Dict[str, Any]
+    properties: dict[str, Any]
     """ The @properties in this class """
 
-    methods: Dict[str, Any]
+    methods: dict[str, Any]
     """ The methods in this class """
 
+    def exclude(self, regexes: list[str]) -> dict[str, Any]:
+        return {name: method for name, method in self.methods.items()
+                if not _matchAnyRegex(regexes, name)}
+
+
 
 def _isMethodInherited(cls, method: str, mro) -> bool:
     m = getattr(cls, method)
@@ -887,7 +909,7 @@ def isMethodInherited(cls, method: str) -> bool:
     return _isMethodInherited(cls, method, mro)
 
 
-def getClassMembers(cls, exclude: List[str] = None, inherited=True) -> ClassMembers:
+def getClassMembers(cls, exclude: list[str] = None, inherited=True) -> ClassMembers:
     """
     Inspects cls and determines its methods and properties
 
@@ -937,7 +959,7 @@ def isExtensionClass(cls) -> bool:
     return False
 
 
-def getEmbeddedSignature(objname: str, doc: str) -> Opt[str]:
+def getEmbeddedSignature(objname: str, doc: str) -> str:
     """
     Returns the docstring embedded signature
 
@@ -949,7 +971,7 @@ def getEmbeddedSignature(objname: str, doc: str) -> Opt[str]:
         doc: the docstring of the object
 
     Returns:
-        the embedded signature or None if no embedded signature found
+        the embedded signature or an empty string if no embedded signature found
     """
     try:
         line0, rest = doc.split("\n", maxsplit=1)
@@ -957,7 +979,7 @@ def getEmbeddedSignature(objname: str, doc: str) -> Opt[str]:
         line0 = doc
     if re.search(fr"{objname}\(", line0):
         return line0.strip()
-    return None
+    return ''
 
 
 def hasEmbeddedSignature(objname: str, doc: str) -> bool:
@@ -977,14 +999,24 @@ def hasEmbeddedSignature(objname: str, doc: str) -> bool:
     return getEmbeddedSignature(objname, doc) is not None
 
 
-def _splitFirstLine(s: str) -> Tuple[str, str]:
+def _splitFirstLine(s: str) -> tuple[str, str]:
+    """
+    Split s into its first line and the rest
+
+    Args:
+        s: the text to split
+
+    Returns:
+        a tuple (firstline: str, rest: str)
+    """
     if "\n" in s:
         l0, l1 = s.split("\n", maxsplit=1)
-        return l0, l1
+        return l0.strip(), l1.strip()
     return s, ''
 
 
-def getModuleMembers(module, exclude: List[str] = None, classesFirst=True) -> Dict[str, Any]:
+def getModuleMembers(module, exclude: list[str] = None, classesFirst=True
+                     ) -> dict[str, Any]:
     """
     Returns a dictionary of {membername:member} in order of appearence
 
@@ -1000,11 +1032,18 @@ def getModuleMembers(module, exclude: List[str] = None, classesFirst=True) -> Di
         a dictionary of all the members defined in this module
 
     """
+
     members = inspect.getmembers(module)
 
     if exclude:
+        logger.info(f"Excluding patterns: {exclude}")
+        excluded = [name for name, item in members if _matchAnyRegex(exclude, name)]
+        if excluded:
+            logger.info(f"Excluded members: {excluded}")
+        else:
+            logger.info(f"No excluded members with patterns: {exclude}")
         members = [(name, item) for name, item in members
-                   if _matchAnyRegex(exclude, name)]
+                   if name not in excluded]
 
     if classesFirst:
         members.sort(key=lambda item: int(inspect.isclass(item)))
@@ -1015,7 +1054,7 @@ def getModuleMembers(module, exclude: List[str] = None, classesFirst=True) -> Di
     return ownmembers
 
 
-def externalModuleMembers(module, include_private=False) -> List[str]:
+def externalModuleMembers(module, include_private=False) -> list[str]:
     """
     Returns a list of member names which appear in dir(module) but are not
     defined there
@@ -1035,7 +1074,7 @@ def externalModuleMembers(module, include_private=False) -> List[str]:
     return external
 
 
-def groupMembers(members: Dict[str, Any]) -> Tuple[dict, dict, dict]:
+def groupMembers(members: dict[str, Any]) -> tuple[dict, dict, dict]:
     """
     Sorts the members into three groups: functions, classes and modules
 
@@ -1058,7 +1097,7 @@ def groupMembers(members: Dict[str, Any]) -> Tuple[dict, dict, dict]:
     return funcs, clss, modules
 
 
-def sortClassesByTree(classes: List[type]) -> List[type]:
+def sortClassesByTree(classes: list[type]) -> list[type]:
     """
     Sort classes according to their inheritance tree
 
@@ -1097,9 +1136,50 @@ def sortClassesByTree(classes: List[type]) -> List[type]:
     return coll
 
 
-def generateDocsForModule(module, renderConfig:RenderConfig=None, exclude: List[str] = None,
-                          startLevel=1, grouped=False, title:str=None,
-                          includeCustomExceptions=False) -> str:
+def _abbreviateSignature(sig: str, maxlen=30) -> str:
+    if len(sig) > maxlen:
+        sig = sig[:maxlen] + '…'
+    return sig
+
+
+def generateModuleTOC(members: dict[str, Any]) -> str:
+    classes = {name: member for name, member in members.items()
+               if inspect.isclass(member)}
+    funcs = {name: member for name, member in members.items()
+             if name not in classes}
+    blocks = []
+    if classes:
+        blocks.append("| Class  | Description  |")
+        blocks.append("| :----  | :----------- |")
+        for name, cls in classes.items():
+            docdef = parseDef(cls)
+            descr = docdef.shortDescr if docdef else '-'
+
+            # blocks.append(f"| `{name}` | {descr} |")
+            blocks.append(f"|  [{name}](#{name.lower()}) | {descr} |")
+        blocks.append("")
+
+    if funcs:
+        blocks.append("| Function  | Description  |")
+        blocks.append("| :-------  | :----------- |")
+        for name, func in funcs.items():
+            docdef = parseDef(func)
+            descr = docdef.shortDescr if docdef else '-'
+            blocks.append(f"| `{name}` | {descr} |")
+        blocks.append("")
+
+    return "\n".join(blocks)
+
+
+def generateDocsForModule(module, 
+                          renderConfig: RenderConfig = None, 
+                          exclude: list[str] = None,
+                          indentLevel=1, 
+                          grouped=False, 
+                          title='',
+                          includeCustomExceptions=False,
+                          toc=True
+                          ) -> str:
     """
     Generate documentation for the given module
 
@@ -1107,8 +1187,8 @@ def generateDocsForModule(module, renderConfig:RenderConfig=None, exclude: List[
         module: the module to generate documentation for
         renderConfig: a RenderConfig
         exclude: functions/classes to exclude. A list of regexes
-        startLevel: heading start level
-        grouped: if True, classes / functions are grouped together. Otherwise the order
+        indentLevel: heading start level
+        grouped: if True, classes / functions are grouped together. Otherwise, the order
             of appearance within the source code is used
         title: if given, it will be used instead of the module name
         includeCustomExceptions: if True, include user defined exceptions into the documentation
@@ -1121,11 +1201,11 @@ def generateDocsForModule(module, renderConfig:RenderConfig=None, exclude: List[
     sep = "\n---------\n"
     blocks = []
     if title:
-        blocks.append(markdownHeader(title, startLevel))
+        blocks.append(markdownHeader(title, indentLevel))
     else:
-        blocks.append(markdownHeader(module.__name__, startLevel))
+        blocks.append(markdownHeader(module.__name__, indentLevel))
     if module.__doc__:
-        doc = markdownReplaceHeadings(module.__doc__, startLevel=startLevel+1)
+        doc = markdownReplaceHeadings(module.__doc__, indentLevel=indentLevel+1)
         blocks.append(doc)
     blocks.append(sep)
     if grouped:
@@ -1142,14 +1222,19 @@ def generateDocsForModule(module, renderConfig:RenderConfig=None, exclude: List[
     if not includeCustomExceptions and classmembers:
         classmembers = [cls for cls in classmembers
                         if Exception not in cls.mro()]
+        
+    if toc:
+        toctext = generateModuleTOC(membersd)
+        blocks.append(toctext)
+        blocks.append(sep)
 
     for cls in classmembers:
-        blocks.append(generateDocsForClass(cls, renderConfig=renderConfig, startLevel=startLevel+1))
+        blocks.append(generateDocsForClass(cls, renderConfig=renderConfig, indentLevel=indentLevel + 1))
         blocks.append(sep)
 
     for func in funcmembers:
         parsed = parseDef(func)
-        doc = renderDocumentation(parsed, renderConfig=renderConfig, startLevel=startLevel+1)
+        doc = renderDocumentation(parsed, renderConfig=renderConfig, indentLevel=indentLevel + 1)
         blocks.append(doc)
         blocks.append(sep)
 
@@ -1158,7 +1243,7 @@ def generateDocsForModule(module, renderConfig:RenderConfig=None, exclude: List[
     return "\n\n".join(blocks)
 
 
-def _matchAnyRegex(patterns: List[str], s: str) -> bool:
+def _matchAnyRegex(patterns: list[str], s: str) -> bool:
     """
     Returns True if s matches any of the regexes in patterns
 
@@ -1174,8 +1259,55 @@ def _matchAnyRegex(patterns: List[str], s: str) -> bool:
     return any(re.search(patt, s) for patt in patterns)
 
 
-def generateDocsForClass(cls, renderConfig:RenderConfig, exclude: List[str] = None,
-                         startLevel=1) -> str:
+@cache
+def parseClassDocstring(cls) -> ParsedDef | None:
+    """
+    Extract the docs of a class to a ParsedDef
+
+    Returns None if the class has no __doc__
+    """
+    if not cls.__doc__:
+        return None
+
+    clsdocs = textwrap.dedent(cls.__doc__)
+    if isExtensionClass(cls) and hasEmbeddedSignature(cls.__qualname__, clsdocs):
+        signature = getEmbeddedSignature(cls.__qualname__, clsdocs)
+        _, clsdocs = _splitFirstLine(clsdocs)
+        clsdocs = textwrap.dedent(clsdocs)
+    else:
+        signature = ''
+    parsedDocs = parseDocstring(clsdocs)
+    parsedDocs.embeddedSignature = signature
+    return parsedDocs
+
+
+def renderClassDocstring(cls,
+                         renderConfig: RenderConfig,
+                         indentLevel=1
+                         ) -> str:
+    """
+    Render the documentation for a given class
+
+    Args:
+        renderConfig: the RenderConfig
+        indentLevel: indentation level
+
+    Returns:
+        the rendered text in the format specified by renderConfig
+    """
+    parsedDocs = parseClassDocstring(cls)
+    if not parsedDocs:
+        return ''
+    return renderDocumentation(parsedDocs,
+                               renderConfig=renderConfig,
+                               indentLevel=indentLevel + 1)
+
+
+def generateDocsForClass(cls,
+                         renderConfig: RenderConfig,
+                         exclude: list[str] = None,
+                         indentLevel=1,
+                         ) -> str:
     """
     Generate documentation for the given class
 
@@ -1183,55 +1315,65 @@ def generateDocsForClass(cls, renderConfig:RenderConfig, exclude: List[str] = No
         cls: the cls to generate documentation for
         renderConfig: a RenderConfig
         exclude: functions/classes to exclude
-        startLevel: heading start level
+        indentLevel: heading start level
         
     Returns:
-        The rendered documentation as a markdown string
+        The rendered documentation as a string in the format specified by the
+        renderConfig
     """
     sep = "\n---------\n"
     classMembers = getClassMembers(cls, inherited=renderConfig.includeInheritedMethods)
-    blocks = [markdownHeader(cls.__qualname__, startLevel)]
+    blocks = [markdownHeader(cls.__qualname__, indentLevel)]
     mro = cls.mro()
     if len(mro) > 2:
         base = mro[1].__qualname__
-        blocks.append(f' - Base Class: [{base}](#{base.lower()})')
+        blocks.append(f' - Base Class: [{base}](#{base.lower()})\n')
     if cls.__doc__:
-        clsdocs = textwrap.dedent(cls.__doc__)
-        if isExtensionClass(cls) and hasEmbeddedSignature(cls.__qualname__, clsdocs):
-            signature = getEmbeddedSignature(cls.__qualname__, clsdocs)
-            _, clsdocs = _splitFirstLine(clsdocs)
-            clsdocs = textwrap.dedent(clsdocs)
-        else:
-            signature = ''
-        parsedDocs = parseDocstring(clsdocs)
-        parsedDocs.embeddedSignature = signature
-        rendered = renderDocumentation(parsedDocs, renderConfig=renderConfig,
-                                       startLevel = startLevel+1)
-        blocks.append(rendered)
+        blocks.append(renderClassDocstring(cls, renderConfig=renderConfig, indentLevel=indentLevel))
+        blocks.append('')
+
+    propertyDefs = {name: parseDef(prop) for name, prop in classMembers.properties.items()}
+
+    methods = classMembers.methods if not exclude else classMembers.exclude(exclude)
+    methodDefs = {name: parseDef(method) for name, method in methods.items()}
+
+    if renderConfig.generateClassTOC and (classMembers.properties or methods):
+        blocks.append(sep)
+        blocks.append('\n**Summary**\n\n')
+        if classMembers.properties:
+            blocks.append('')
+            blocks.append("| Property  | Description  |")
+            blocks.append("| :-------- | :----------- |")
+            for name, propDef in propertyDefs.items():
+                blocks.append(f"| {name} | {propDef.shortDescr or '-'} |")
+            blocks.append('')
+
+        if methods:
+            blocks.append('')
+            blocks.append("| Method  | Description  |")
+            blocks.append("| :------ | :----------- |")
+            for name, methDef in methodDefs.items():
+                blocks.append(f"| [{name}](#{name}) | {methDef.shortDescr or '-'} |")
+            blocks.append('')
+        blocks.append(sep)
 
     if classMembers.properties:
-        blocks.append("**Attributes**")
-        # blocks.append(markdownHeader("Attributes", startLevel+1))
-        for prop in classMembers.properties.values():
-            p = parseDef(prop)
-            doc = renderDocumentation(p, renderConfig=renderConfig, startLevel=startLevel+2)
+        blocks.append("\n**Attributes**\n")
+        for propname, propdef in propertyDefs.items():
+            doc = renderDocumentation(propdef, renderConfig=renderConfig, indentLevel=indentLevel + 2)
             blocks.append(doc)
+        blocks.append('')
 
-    if classMembers.methods:
-        if exclude:
-            methods = [method for methodname, method in classMembers.methods.items()
-                       if not _matchAnyRegex(exclude, methodname)]
-        else:
-            methods = list(classMembers.methods.values())
-
+    if methods:
         blocks.append(sep)
-        blocks.append("**Methods**")
-        methodDocs = generateDocsForFunctions(methods,
+        blocks.append("\n**Methods**\n")
+        methodDocs = generateDocsForFunctions(list(methods.values()),
                                               renderConfig=renderConfig,
-                                              startLevel=startLevel+1)
+                                              indentLevel=indentLevel + 1)
         blocks.append(methodDocs)
+        blocks.append('')
 
-    docs = "\n\n".join(blocks)
+    docs = "\n".join(blocks)
     return docs
 
 
@@ -1242,10 +1384,11 @@ def generateMkdocsIndex(projectName: str,
                         url:str="<Project URL>",
                         longDescription:str="<Long Description>",
                         quickStart:str="<Quick Start>",
-                        includeWelcome=True
+                        includeWelcome=True,
+                        fmt='markdown'
                         ) -> str:
     """
-    Generate the template for an index.md file suitable for mkdocs
+    Generate the template for an index file suitable for mkdocs
 
     Args:
         projectName: the name of the project
@@ -1259,6 +1402,9 @@ def generateMkdocsIndex(projectName: str,
     Returns:
         a string which could be saved to an index.md file
     """
+    if fmt != 'markdown':
+        raise ValueError(f"At the moment only markdown is supported, got {fmt=}")
+
     if includeWelcome:
         welcomeStr = f"Welcome to the **{projectName}** documentation!"
     else:
