@@ -99,14 +99,12 @@ def pngRemoveTransparency(pngfile: str, outfile='', background=(255, 255, 255)
     if outfile:
         img.save(outfile)
     else:
-        # instead of overwriting, we send the old file to trash
-        import send2trash
-        send2trash.send2trash(pngfile)
+        os.remove(pngfile)
         img.save(pngfile)
 
 
-def readImageAsBase64(imgpath: str, 
-                      outformat='', 
+def readImageAsBase64(imgpath: str,
+                      outformat='',
                       removeAlpha=False
                       ) -> tuple[bytes, int, int]:
     """
@@ -123,8 +121,11 @@ def readImageAsBase64(imgpath: str,
     """
     import base64
     if not outformat:
-        outformat = os.path.splitext(imgpath)[1][1:].lower()
-    assert outformat in {'jpeg', 'png'}
+        ext = os.path.splitext(imgpath)[1][1:].lower()
+        outformat = {'png': 'png', 'jpg': 'jpeg', 'jpeg': 'jpeg'}.get(ext)
+        if outformat is None:
+            raise ValueError(f"Format unknown for file {imgpath}")
+    assert outformat in {'jpeg', 'png'}, f"Unsupported format: {outformat}. Formats supported: png, jpeg"
     buffer = BytesIO()
     im = Image.open(imgpath)
     if removeAlpha:
@@ -135,7 +136,7 @@ def readImageAsBase64(imgpath: str,
     return imgbytes, width, height
 
 
-def cropToBoundingBox(inputpath: str, outpath: str = '', margin: Union[int, tuple[int, int, int, int]] = 0) -> str | None:
+def cropToBoundingBox(inputpath: str, outpath: str = '', margin: int | tuple[int, int, int, int] = 0) -> None:
     """
     Crop an image to its content, trimming any empty space
 
@@ -153,7 +154,7 @@ def cropToBoundingBox(inputpath: str, outpath: str = '', margin: Union[int, tupl
     box = diff.getbbox()
 
     if not box:
-        return 'Could not find bounding box'
+        raise RuntimeError('Could not find bounding box')
 
     if margin:
         if isinstance(margin, int):
@@ -171,18 +172,18 @@ def cropToBoundingBox(inputpath: str, outpath: str = '', margin: Union[int, tupl
 
 
 def htmlImgBase64(imgpath: str,
-                  width: Union[int, str] = None, 
-                  maxwidth: Union[int,str] = None,
-                  margintop='14px', 
+                  width: int | str = None,
+                  maxwidth: int | str = None,
+                  margintop='14px',
                   padding='10px',
-                  removeAlpha=False, 
+                  removeAlpha=False,
                   scale=1.
                   ) -> str:
     """
     Read an image and return the data as base64 within an img html tag
-    
+
     Args:
-        imgpath: the path to the image 
+        imgpath: the path to the image
         width: the width of the displayed image. Either a width
             in pixels or a str as passed to css ('800px', '100%').
         maxwidth: similar to width
@@ -196,7 +197,7 @@ def htmlImgBase64(imgpath: str,
                                                   removeAlpha=removeAlpha)
     imgstr = imgstr.decode('utf-8')
     if scale and not width:
-        width = imwidth * scale
+        width = int(imwidth * scale)
     attrs = [f'padding:{padding}',
              f'margin-top:{margintop}']
     if maxwidth:
