@@ -4,11 +4,12 @@ More itertools
 from __future__ import annotations
 
 import sys
-from itertools import *
+import itertools as it
 import operator as _operator
 import collections as _collections
 import random as _random
 from math import inf
+import warnings
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
     from typing import Iterable, Iterator, TypeVar, Callable, Sequence
@@ -16,7 +17,7 @@ if TYPE_CHECKING:
     T2 = TypeVar("T2")
 
 # ----------------------------------------------------------------------------
-# 
+#
 #    protocol: seq, other parameters
 #
 #    Exception: functions that match the map protocol: func(predicate, seq)
@@ -26,7 +27,7 @@ if TYPE_CHECKING:
 
 def take(seq: Iterable[T], n:int) -> list[T]:
     """returns the first n elements of seq as a list"""
-    return list(islice(seq, n))
+    return list(it.islice(seq, n))
 
 
 def last(seq: Iterable[T]) -> T | None:
@@ -35,7 +36,7 @@ def last(seq: Iterable[T]) -> T | None:
 
     if *seq* is an iterator, it will consume it
     """
-    if isinstance(seq, _collections.Sequence):
+    if hasattr(seq, '__getitem__'):
         try:
             return seq[-1]
         except IndexError:
@@ -47,8 +48,8 @@ def last(seq: Iterable[T]) -> T | None:
         return x
 
 
-def first(it: Iterable[T], default=None) -> T | None:
-    return next(it, default)
+def first(seq: Iterable[T], default=None) -> T | None:
+    return next(seq, default)
 
 
 def consume(iterator, n: int) -> None:
@@ -64,9 +65,9 @@ def consume(iterator, n: int) -> None:
         _collections.deque(iterator, maxlen=0)
     else:
         # advance to the empty slice starting at position n
-        next(islice(iterator, n, n), None)
+        next(it.islice(iterator, n, n), None)
+
 
-        
 def drop(seq: Iterable[T], n: int) -> Iterable[T]:
     """
     return an iterator over seq with n elements consumed
@@ -77,12 +78,12 @@ def drop(seq: Iterable[T], n: int) -> Iterable[T]:
         >>> list(drop(range(10), 3))
         [3, 4, 5, 6, 7, 8, 9]
     """
-    return islice(seq, n, None)
+    return it.islice(seq, n, None)
 
 
 def nth(seq: Iterable[T], n: int) -> T:
     """Returns the nth item"""
-    return next(islice(seq, n, n+1))
+    return next(it.islice(seq, n, n+1))
 
 
 def pad(seq: Iterable[T], element:T2=None) -> Iterator[T | T2]:
@@ -90,16 +91,16 @@ def pad(seq: Iterable[T], element:T2=None) -> Iterator[T | T2]:
     Returns the elements in *seq* and then return *element* indefinitely.
 
     Useful for emulating the behavior of the built-in map() function.
-    
+
     >>> take(pad(range(10), "X"), 15)
     [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 'X', 'X', 'X', 'X', 'X']
     """
-    return chain(seq, repeat(element))
+    return it.chain(seq, it.repeat(element))
 
 
 def ncycles(seq: Iterable[T], n: int) -> Iterable[T]:
     """Returns the sequence elements n times"""
-    return chain(*repeat(seq, n))
+    return it.chain(*it.repeat(seq, n))
 
 
 class Accum:
@@ -115,8 +116,8 @@ class Accum:
         >>> takewhile((lambda item, accum=Accum(): accum(item) < 100), seq)
     """
 
-    def __init__(self, init:T=0):
-        self.value:T = init
+    def __init__(self, init: T = 0):
+        self.value: T = init
 
     def __call__(self, value:T) -> T:
         self.value += value
@@ -146,22 +147,24 @@ def repeatfunc(func, times=None, *args):
     Example:  repeatfunc(random.random)
     """
     if times is None:
-        return starmap(func, repeat(args))
+        return it.starmap(func, it.repeat(args))
     else:
-        return starmap(func, repeat(args, times))
+        return it.starmap(func, it.repeat(args, times))
 
 
-def pairwise(iterable: Iterable[T]) -> zip[tuple[T, T]]:
+def _pairwise(iterable: Iterable[T]) -> zip[tuple[T, T]]:
     """
     Similar to window(seq, size=2, step=1)
-    
+
+    Pairwise exists since 3.10
+
     Example
     ~~~~~~~
 
         >>> list(pairwise(range(4)))
         [(0, 1), (1, 2), (2, 3)]
-    """    
-    a, b = tee(iterable)
+    """
+    a, b = it.tee(iterable)
     try:
         next(b)
     except StopIteration:
@@ -183,7 +186,7 @@ def window(iterable: Iterable[T], size=3, step=1) -> Iterator[tuple[T, ...]]:
 
     Example
     ~~~~~~~
-    
+
     >>> seq = range(6)
     >>> list(window(seq, 3, 1))
     [(0, 1, 2), (1, 2, 3), (2, 3, 4), (3, 4, 5)]
@@ -193,16 +196,69 @@ def window(iterable: Iterable[T], size=3, step=1) -> Iterator[tuple[T, ...]]:
     >>> list(window(range(5), 2, 1))
     [(0, 1), (1, 2), (2, 3), (3, 4)]
     """
-    iterators = tee(iterable, size)
+    iterators = it.tee(iterable, size)
     for skip_steps, itr in enumerate(iterators):
-        for _ in islice(itr, skip_steps):
+        for _ in it.islice(itr, skip_steps):
             pass
     winiter = zip(*iterators)
     if step != 1:
-        winiter = islice(winiter, 0, 99999999, step)
+        winiter = it.islice(winiter, 0, 99999999, step)
     return winiter
 
 
+def window_padded(seq: Iterable[T], size=3, step=1, paddingval=None) -> Iterator[tuple[T, ...]]:
+    """Similar to window, but the last window will be padded
+
+    Args:
+        seq: the sequence
+        size: the size of each window
+        step: how much to advance the window each iteration
+        paddingval: value used to pad the last iteration if there are not enough items
+
+    Returns:
+        an iterator of tuples, where each tuple has *size* elements
+
+    Example
+    ~~~~~~~
+
+    >>> list(windowed([1, 2, 3, 4, 5, 6], 3, 2))
+    [(1, 2, 3), (3, 4, 5), (5, None, None)]
+
+    """
+    if size < 0:
+        raise ValueError('n must be >= 0')
+
+    if size == 0:
+        yield ()
+        return
+
+    if step < 1:
+        raise ValueError('step must be >= 1')
+
+    iterator = iter(seq)
+
+    # Generate first window
+    window = _collections.deque(it.islice(iterator, size), maxlen=size)
+
+    # Deal with the first window not being full
+    if not window:
+        return
+
+    if len(window) < size:
+        yield tuple(window) + ((paddingval,) * (size - len(window)))
+        return
+    yield tuple(window)
+
+    # Create the filler for the next windows. The padding ensures
+    # we have just enough elements to fill the last window.
+    padding = (paddingval,) * (size - 1 if step >= size else step - 1)
+    filler = map(window.append, it.chain(iterator, padding))
+
+    # Generate the rest of the windows
+    for _ in it.islice(filler, step - 1, None, step):
+        yield tuple(window)
+
+
 def window_fixed_size(seq: Iterable[T], size: int, maxstep: int
                            ) -> Iterable[tuple[T, ...]]:
     """
@@ -219,7 +275,7 @@ def window_fixed_size(seq: Iterable[T], size: int, maxstep: int
         elements if they don't fit evenly in the window size
 
     Example
-    =======
+    ~~~~~~~
 
     >>> list(window_fixed_size(range(10)), 5,2)
     [[0, 1, 2, 3, 4], [2, 3, 4, 5, 6], [4, 5, 6, 7, 8], [5, 6, 7, 8, 9]]
@@ -264,7 +320,7 @@ def iterchunks(seq, chunksize: int) -> Iterable[tuple]:
     [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9, 10, 11), (12, 13, 14), (15, 16, 17), (18, 19)]
     """
 
-    padded = chain(seq, repeat(_Sentinel))
+    padded = it.chain(seq, it.repeat(_Sentinel))
     for chunk in window(padded, size=chunksize, step=chunksize):
         if chunk[-1] is _Sentinel:
             if chunk[0] is _Sentinel:
@@ -310,7 +366,7 @@ def isiterable(obj, exclude=(str,)) -> bool:
     if exclude:
         return hasattr(obj, '__iter__') and (not isinstance(obj, exclude))
     return hasattr(obj, '__iter__')
-    
+
 
 def random_combination(iterable: Iterable, r):
     """Random selection from itertools.combinations(iterable, r)"""
@@ -338,6 +394,7 @@ def partialsum(seq: Iterable[T], start=0) -> Iterable[T]:
         n2 -> n0 + n1 + n2
         n3 -> n0 + n1 + n2 + n3
     """
+    warnings.warn("Use itertools.accumulate instead", DeprecationWarning, stacklevel=2)
     accum = start
     for i in seq:
         accum += i
@@ -359,7 +416,7 @@ def partialmul(seq: Iterable[T], start=1) -> Iterable[T]:
         accum *= i
         yield accum
 
-        
+
 def partialavg(seq: Iterable[T]) -> T:
     """
     Return the partial average in seq
@@ -376,7 +433,7 @@ def ilen(seq: Iterable) -> int:
     """
     Consume an iterable not reading it into memory; return the number of items.
     """
-    counter = count()
+    counter = it.count()
     _collections.deque(zip(seq, counter), maxlen=0)  # (consume at C speed)
     return next(counter)
 
@@ -401,7 +458,7 @@ def avg(seq: Iterable[T], empty=0) -> T:
         i += 1
     return accum / i if i else empty
 
-        
+
 def flatten(s: Iterable[T | Iterable[T]], exclude=(str,), levels=inf) -> Iterator[T]:
     """
     Return an iterator to the flattened items of sequence s
@@ -427,7 +484,7 @@ def flatten(s: Iterable[T | Iterable[T]], exclude=(str,), levels=inf) -> Iterato
             yield item
         else:
             yield from flattened(item, exclude, levels-1)
-    
+
 
 def flatdict(d: dict) -> list:
     """
@@ -470,7 +527,7 @@ def flattened(s: Iterable[T | Iterable[T]], exclude=(str,), levels: int = None,
         out = []
     _flattened2(s, out, exclude, levels=levels)
     return out
-    
+
 
 def _flattened2(s, out: list, exclude, levels: int = None) -> None:
     for item in s:
@@ -482,7 +539,7 @@ def _flattened2(s, out: list, exclude, levels: int = None) -> None:
 
 def flattenonly(l, types):
     """
-    Flatten only if subsequences are of type 'type' 
+    Flatten only if subsequences are of type 'type'
 
     Args:
         l: a seq
@@ -504,14 +561,14 @@ def flattenonly(l, types):
         else:
             yield elem
 
-            
+
 def zipflat(*seqs):
     """
     like izip but flat. It has the same effect as `flatten(zip(seqA, seqB), levels=1)`
-    
+
     Example
     -------
-    
+
     >>> list(zipflat(range(5), "ABCDE"))
     [0, 'A', 1, 'B', 2, 'C', 3, 'D', 4, 'E']
     """
@@ -519,11 +576,11 @@ def zipflat(*seqs):
         for elem in elems:
             yield elem
 
-            
+
 def butlast(seq: Iterable[T]) -> Iterator[T]:
     """
     iterate over seq[:-1]
-    
+
     >>> list(butlast(range(5)))
     [0, 1, 2, 3]
     """
@@ -533,7 +590,7 @@ def butlast(seq: Iterable[T]) -> Iterator[T]:
         yield lastitem
         lastitem = i
 
-        
+
 def butn(seq: Iterable[T], n: int) -> Iterable[T]:
     """
     iterate over seq[:-n]
@@ -545,7 +602,7 @@ def butn(seq: Iterable[T], n: int) -> Iterable[T]:
         yield d.popleft()
         d.append(x)
 
-        
+
 def intercalate(seq: Iterable[T], item:T2) -> Iterator[T|T2]:
     """
     Intercalate *item* between elements of *seq*
@@ -556,7 +613,7 @@ def intercalate(seq: Iterable[T], item:T2) -> Iterator[T|T2]:
     >>> list(intercalate(range(5), "X"))
     [0, 'X', 1, 'X', 2, 'X', 3, 'X', 4]
     """
-    return butlast(zipflat(seq, repeat(item)))
+    return butlast(zipflat(seq, it.repeat(item)))
 
 
 def partialreduce(seq: Iterable[T], func: Callable, start=0) -> Iterator[T]:
@@ -574,7 +631,7 @@ def partialreduce(seq: Iterable[T], func: Callable, start=0) -> Iterator[T]:
         except StopIteration:
             raise StopIteration
 
-        
+
 def mesh(xs: Iterable[T], ys: Iterable[T2]) -> Iterator[tuple[T, T2]]:
     """
     iterator over the lexicographical pairs
@@ -607,7 +664,7 @@ def mesh(xs: Iterable[T], ys: Iterable[T2]) -> Iterator[tuple[T, T2]]:
         for y in ys:
             yield (x, y)
 
-            
+
 def mesh3(A, B, C):
     """
     the same as mesh, but over 3 iterators
@@ -617,7 +674,7 @@ def mesh3(A, B, C):
             for c in C:
                 yield (a, b, c)
 
-                
+
 def unique(seq: Iterable[T]) -> Iterator[T]:
     """
     Return only unique elements of a sequence (keeps order)
@@ -639,9 +696,9 @@ def unique(seq: Iterable[T]) -> Iterator[T]:
             seen.add(item)
             yield item
 
-            
+
 def interleave(seqs, pass_exceptions=()):
-    """ 
+    """
     Interleave a sequence of sequences
 
     >>> list(interleave([[1, 2], [3, 4]]))
@@ -664,10 +721,18 @@ def interleave(seqs, pass_exceptions=()):
             except (StopIteration,) + tuple(pass_exceptions):
                 pass
         iters = newiters
-        
 
-def chunked(iterable, n):
-    """Break *iterable* into lists of length *n*:
+
+def chunked(iterable, n: int, strict=False):
+    """Break *iterable* into lists of length *n*
+
+
+    Args:
+        iterable: the iterable to chunk
+        n: size of each chunk
+        strict: raises an error if iterable is not divisible by n
+
+    Example::
 
         >>> list(chunked([1, 2, 3, 4, 5, 6], 3))
         [[1, 2, 3], [4, 5, 6]]
@@ -685,6 +750,7 @@ def chunked(iterable, n):
     list is yielded.
 
     """
+    from functools import partial
     iterator = iter(partial(take, n, iter(iterable)), [])
     if strict:
         if n is None:
