@@ -9,25 +9,26 @@ Miscellaneous math utilities
 
 """
 from __future__ import annotations
+
 import operator as _operator
 import random as _random
-from functools import reduce
-from math import gcd, sqrt, cos, sin, radians, ceil, hypot, pi, asin, floor, factorial, e
 import sys as _sys
+import functools as _functools
+import math
 import numpy as np
-from numbers import Rational, Number
-from typing import TYPE_CHECKING, TypeVar, Sequence, Callable
-
 
 try:
     from quicktions import Fraction
 except ImportError:
     from fractions import Fraction
 
+from typing import TYPE_CHECKING
+
 if TYPE_CHECKING:
-    from typing import Union, Iterator
-    number_t = Union[Rational, float]
-    T = TypeVar("T", bound=number_t)
+    from typing import Iterator, Callable, Sequence, TypeVar, TypeAlias, Union
+    from fractions import Fraction
+    number_t: TypeAlias = Union[Fraction, float]
+    T = TypeVar("T", float, Fraction)
     T2 = TypeVar("T2", bound=number_t)
 
 
@@ -63,7 +64,8 @@ __all__ = ("PHI",
            )
 
 
-class NotFoundError(ValueError): pass
+class NotFoundError(ValueError):
+    pass
 
 
 # phi, in float (double) form and as Rational number with a precission of 2000
@@ -94,7 +96,33 @@ def intersection(u1: T, u2: T, v1: T, v2: T) -> tuple[T | None, T]:
     """
     x0 = u1 if u1 > v1 else v1
     x1 = u2 if u2 < v2 else v2
-    return (x0, x1) if x0 < x1 else (None, 0)
+    return (x0, x1) if x0 < x1 else (None, 0.)
+
+
+def overlap(u1: T, u2: T, v1: T, v2: T) -> tuple[T, T] | None:
+    """
+    return the intersection of (u1, u2) and (v1, v2) or None if no intersection
+
+    Args:
+        u1: lower bound of range U
+        u2: higher bound of range U
+        v1: lower bound of range V
+        v2: higher bound of range V
+
+    Returns:
+        the intersection between range U and range V as a tuple (start, end). If no intersection is found,
+        start will be None.
+
+    Example::
+
+        >>> if intersect := overlap(0, 3, 2, 5):
+        ...     x0, x1 = intersect
+        ...     # there is an intersection
+
+    """
+    x0 = u1 if u1 > v1 else v1
+    x1 = u2 if u2 < v2 else v2
+    return (x0, x1) if x0 < x1 else None
 
 
 def hasintersect(u1: T, u2: T, v1: T, v2: T) -> bool:
@@ -135,29 +163,25 @@ def frange(start: float, stop: float = None, step: float = 0.) -> Iterator[float
         yield start + step*i
 
 
-def asFraction(x) -> Fraction:
+def asfraction(x) -> Fraction:
     """ Convert x to a Fraction if it is not already one """
-    if isinstance(x, Fraction):
-        return x
-    elif isinstance(x, Rational):
-        return Fraction(x.numerator, x.denominator)
-    return Fraction(x)
+    return x if isinstance(x, Fraction) else Fraction(x)
 
 
 def fraction_range(start: number_t, stop: number_t = None, step: number_t = None
                    ) -> Iterator[Fraction]:
     """ Like range, but yielding Fractions """
     if stop is None:
-        stopF = asFraction(start)
+        stopF = asfraction(start)
         startF = Fraction(0)
     else:
-        startF = asFraction(start)
-        stopF = asFraction(stop)
+        startF = asfraction(start)
+        stopF = asfraction(stop)
 
     if step is None:
         step = Fraction(1)
     else:
-        step = asFraction(step)
+        step = asfraction(step)
     while startF < stopF:
         yield startF
         startF += step
@@ -169,14 +193,14 @@ def linspace(start: float, stop: float, numitems: int) -> list[float]:
     return [start + dx*i for i in range(numitems)]
 
 
-def linlin(x: T, x0:T, x1:T, y0:T, y1:T) -> T:
+def linlin(x: T, x0: T, x1: T, y0: T, y1: T) -> T:
     """
     Convert x from range x0-x1 to range y0-y1
     """
     return (x - x0) * (y1 - y0) / (x1-x0) + y0
 
 
-def clip(x:T, minvalue:T, maxvalue:T) -> T:
+def clip(x: T, minvalue: T, maxvalue: T) -> T:
     """
     clip the value of x between minvalue and maxvalue
     """
@@ -199,9 +223,9 @@ def lcm(*numbers: int) -> int:
 
     """
     def lcm2(a, b):
-        return (a*b) // gcd(a, b)
+        return (a*b) // math.gcd(a, b)
 
-    return reduce(lcm2, numbers, 1)
+    return _functools.reduce(lcm2, numbers, 1)
 
 
 def min_common_denominator(floats: Iterator[float], limit: int = int(1e10)) -> int:
@@ -292,8 +316,8 @@ def euclidian_distance(values: Sequence[float], weights: Sequence[float]=None) -
     """
     if weights:
         s = sum(value*value * weight for value, weight in zip(values, weights))
-        return sqrt(s)
-    return sqrt(sum(value**2 for value in values))
+        return math.sqrt(s)
+    return math.sqrt(sum(value**2 for value in values))
 
 
 def weighted_euclidian_distance(pairs: list[tuple[float, float]]) -> float:
@@ -320,14 +344,14 @@ def prod(numbers: Sequence[number_t]) -> number_t:
 
         x0 * x1 * x2 ... * xn | x in numbers
     """
-    return reduce(_operator.mul, numbers)
+    return _functools.reduce(_operator.mul, numbers)
 
 
 def geometric_mean(numbers: Sequence[number_t]) -> float:
     """
     The geometric mean is often used to find the mean of data measured in different units.
     """
-    return prod(numbers) ** (1/len(numbers))
+    return float(prod(numbers)) ** (1/len(numbers))
 
 
 def harmonic_mean(numbers: Sequence[T]) -> T:
@@ -404,14 +428,13 @@ def derivative(func: Callable[[number_t], number_t], h=0) -> Callable[[number_t]
     return lambda x: (float(func(x+h*1.0j))).imag / h
 
 
-def logrange(start: float, stop: float, num=50, base=10) -> np.ndarray:
+def logrange(start: float, stop: float, num=50, base=10, minstart=1e-12) -> np.ndarray:
     """
     create an array [start, ..., stop] with a logarithmic scale
     """
-    log = np.log
     if start == 0:
-        start = 0.000000000001
-    return np.logspace(log(start, base), log(stop, base), num, base=base)
+        start = minstart
+    return np.logspace(math.log(start, base), math.log(stop, base), num, base=base)
 
 
 def randspace(begin: float, end: float, numsteps: int, include_end=True
@@ -515,7 +538,7 @@ def roundres(x, resolution=1.0):
     return round(x / resolution) * resolution
 
 
-def next_in_grid(x: number_t, step: number_t, offset: number_t = 0) -> float:
+def next_in_grid(x: T, step: T, offset: T) -> T:
     """
     The next value in the grid defined by step/offset which is >= x
 
@@ -524,7 +547,7 @@ def next_in_grid(x: number_t, step: number_t, offset: number_t = 0) -> float:
         step: the step of the grid
         offset: the offset of the grid
 
-    Returns
+    Returns:
         the next value within the grid higher or equal to x
 
     Example
@@ -536,7 +559,12 @@ def next_in_grid(x: number_t, step: number_t, offset: number_t = 0) -> float:
         >>> next_in_grid(1.31, Fraction(1, 3))
         Fraction(4, 3)
     """
-    return offset + ceil((x - offset) / step) * step
+    grididx = math.ceil((x - offset) / step)
+    if isinstance(step, Fraction):
+        return step * Fraction(grididx) + Fraction(offset)
+    else:
+        assert isinstance(step, (int, float)) and isinstance(offset, (int, float))
+        return float(step * grididx + offset)  # type: ignore
 
 
 def modulo_shortest_distance(x: number_t, origin: number_t, mod: int):
@@ -551,9 +579,10 @@ def modulo_shortest_distance(x: number_t, origin: number_t, mod: int):
 
     Calculate the interval between note D5 and B3, independently of octaves
 
+    .. code-block:: python
 
-    >>> interval = modulo_shortest_distance(74, 59, 12)
-    3
+        >>> interval = modulo_shortest_distance(74, 59, 12)
+        3
     """
     xclock = (x - origin) % mod
     xanti = (origin - x) % mod
@@ -579,8 +608,8 @@ def rotate2d(point: tuple[float, float],
     """
     x = point[0] - origin[0]
     yorz = point[1] - origin[1]
-    newx = (x*cos(radians(degrees))) - (yorz*sin(radians(degrees)))
-    newyorz = (x*sin(radians(degrees))) + (yorz*cos(radians(degrees)))
+    newx = (x*math.cos(math.radians(degrees))) - (yorz*math.sin(math.radians(degrees)))
+    newyorz = (x*math.sin(math.radians(degrees))) + (yorz*math.cos(math.radians(degrees)))
     newx += origin[0]
     newyorz += origin[1]
     return newx, newyorz
@@ -591,7 +620,7 @@ def periodic_float_to_fraction(s: str) -> Fraction:
     Convert a float with a periodic part to its fraction
 
     Args:
-        s: the numer as string. Notate the periodic part 
+        s: the numer as string. Notate the periodic part
            (for example 1/3=0.333...)
            as 0.(3, without repetitions. For example, 2.83333... as 2.8(3
 
@@ -732,7 +761,7 @@ def intersection_area_between_circles(x1: float, y1: float, r1: float,
 
     from https://www.xarg.org/2016/07/calculate-the-intersection-area-of-two-circles/
     """
-    d = hypot(x2 - x1, y2 - y1)
+    d = math.hypot(x2 - x1, y2 - y1)
     if d >= r1 + r2:
         return 0
     a = r1 * r1
@@ -741,15 +770,15 @@ def intersection_area_between_circles(x1: float, y1: float, r1: float,
         x = (a - b + d*d) / (2*d)
     else:
         # They share the same center!
-        return pi*min(a, b)
+        return math.pi*min(a, b)
     z = x * x
     if a < z:
         # One circle is embedded in the other?
-        return pi * min(a, b)
-    y = sqrt(a - z)
+        return math.pi * min(a, b)
+    y = math.sqrt(a - z)
     if d <= abs(r2 - r1):
-        return pi * min(a, b)
-    return a * asin(y / r1) + b*asin(y / r2) - y * (x + sqrt(z + b - a))
+        return math.pi * min(a, b)
+    return a * math.asin(y / r1) + b*math.asin(y / r2) - y * (x + math.sqrt(z + b - a))
 
 
 def roman(n: int) -> str:
@@ -780,11 +809,11 @@ def roman(n: int) -> str:
     while n >= div:
         div *= 10
 
-    div /= 10
+    div //= 10
     res = ""
     while n:
         # main significant digit extracted into lastNum
-        lastNum = int(n / div)
+        lastNum = n // div
         if lastNum <= 3:
             res += (romans[div] * lastNum)
         elif lastNum == 4:
@@ -793,8 +822,8 @@ def roman(n: int) -> str:
             res += (romans[div * 5] + (romans[div] * (lastNum - 5)))
         elif lastNum == 9:
             res += (romans[div] + romans[div * 10])
-        n = floor(n % div)
-        div /= 10
+        n = math.floor(n % div)
+        div //= 10
     return res
 
 
@@ -810,9 +839,9 @@ def fractional_factorial(x: float) -> float:
 
     """
     if isinstance(x, int):
-        return factorial(x)
+        return math.factorial(x)
 
-    fact = sqrt(pi)*(x/e)**x
+    fact = math.sqrt(math.pi)*(x/math.e)**x
     fact *= (((8*x + 4)*x + 1)*x + 1/30.)**(1./6.)
     return fact
 
@@ -833,3 +862,68 @@ def nextpowerof2(x) -> int:
     return 1 if x == 0 else 2 ** (x - 1).bit_length()
 
 
+def gini(xs: Sequence[T], eps=1e-12, normalize=True) -> float:
+    """
+    Calculate the Gini coefficient for a given sequence of values.
+
+    The Gini coefficient is a measure of statistical dispersion intended to represent
+    income or wealth distribution within a nation or a social group.
+
+    Args:
+        xs (Sequence[T]): A sequence of values for which the Gini coefficient is to be calculated.
+
+    Returns:
+        The Gini coefficient value. A value of 1 indicates perfect inequality, while a value of 0
+        indicates perfect equality.
+    """
+    # based on bottom eq:
+    # http://www.statsdirect.com/help/generatedimages/equations/equation154.svg
+    # from:
+    # http://www.statsdirect.com/help/default.htm#nonparametric_methods/gini.htm
+    # All values are treated equally, arrays must be 1d:
+    array = np.asarray(xs)
+    array = array.flatten()
+    if np.amin(array) < 0:
+        # Values cannot be negative:
+        array -= np.amin(array)
+    # Values cannot be 0:
+    array = array + eps
+    # Values must be sorted:
+    array = np.sort(array)
+    # Index per array element:
+    index = np.arange(1, array.shape[0]+1)
+    # Number of array elements:
+    n = array.shape[0]
+    # Gini coefficient:
+    index2 = 2 * index - (n + 1)
+    coeff = float((np.sum(index2 * array)) / (n * np.sum(array)))
+    if normalize:
+        maxcoeff = (array.shape[0] * 2 - (n+1)) / n
+        coeff /= maxcoeff
+        if coeff > 1:
+            coeff = 1
+    if coeff < eps:
+        coeff = 0
+    return coeff
+
+
+def exponcurve(num: int, exp: float, x0: float, y0: float, x1: float, y1: float) -> np.ndarray:
+    """
+    Generate an exponential curve between two points
+
+    Args:
+        num: number of points to generate
+        exp: exponent of the curve
+        x0: start x-coordinate
+        y0: start y-coordinate
+        x1: end x-coordinate
+        y1: end y-coordinate
+
+    Returns:
+        a numpy array of shape (num,) containing the y-coordinates of the curve.
+
+    """
+    xs = np.linspace(x0, x1, num)
+    dxs = (xs - x0) / (x1 - x0)
+    ys = (dxs ** exp) * (y1 - y0) + y0
+    return ys
