@@ -14,26 +14,23 @@ Miscellaneous functionality
 from __future__ import annotations
 import os as _os
 import sys as _sys
-import math
 from bisect import bisect as _bisect
-from collections import namedtuple as _namedtuple
 import re as _re
-import dataclasses
 
 import numpy as np
-from fractions import Fraction
-import numbers
-from .envir import inside_jupyter
-from .common import runonce
-from . import iterlib
 
 
 from typing import TYPE_CHECKING
 if TYPE_CHECKING or 'sphinx' in _sys.modules:
-    from typing import TypeVar, Sequence, Union, Optional, Callable, Any, Iterable
+    from typing import TypeVar, Sequence, Union, Callable, Any, Iterable
     T = TypeVar("T")
     T2 = TypeVar("T2")
+    from fractions import Fraction
+    import numbers
+    num_t =  TypeVar("num_t", int, float, numbers.Rational)
     number_t = Union[float, numbers.Rational]
+    import subprocess
+
 
 
 # ------------------------------------------------------------
@@ -70,8 +67,8 @@ def reverse_recursive(seq: list):
     return out
 
 
-def _partialsum(seq: Sequence[T]) -> list[T]:
-    accum = 0
+def _partialsum(seq: Sequence[num_t], init: num_t) -> list[num_t]:
+    accum: num_t = init
     out = []
     for i in seq:
         accum += i
@@ -98,7 +95,7 @@ def wrap_by_sizes(flatseq: list, packsizes: Sequence[int]) -> list[list]:
         [[1, 2, 3], [4, 5, 6, 7, 8], [9, 10]]
 
     """
-    offsets = [0] + _partialsum(packsizes)
+    offsets = [0] + _partialsum(packsizes, 0)
     start = offsets[0]
     out = []
     for i in range(1, len(offsets)):
@@ -115,7 +112,7 @@ def wrap_by_sizes(flatseq: list, packsizes: Sequence[int]) -> list[list]:
 # ------------------------------------------------------------
 
 
-def nearest_element(item: float, seq: Union[list[float], np.ndarray]) -> float:
+def nearest_element(item: float, seq: list[float] | np.ndarray) -> float:
     """
     Find the nearest element (the element, not the index) in seq
 
@@ -162,7 +159,7 @@ def nearest_element(item: float, seq: Union[list[float], np.ndarray]) -> float:
     return element_l
 
 
-def nearest_unsorted(x: number_t, seq: list[number_t]) -> number_t:
+def nearest_unsorted(x: num_t, seq: list[num_t]) -> num_t:
     """
     Find nearest item in an unsorted sequence
 
@@ -185,7 +182,7 @@ def nearest_unsorted(x: number_t, seq: list[number_t]) -> number_t:
     return min((abs(x - y), y) for y in seq)[1]
 
 
-def nearest_index(item: number_t, seq: Sequence[number_t]) -> int:
+def nearest_index(item: num_t, seq: Sequence[num_t]) -> int:
     """
     Return the index of the nearest element in seq to item
 
@@ -359,7 +356,7 @@ def firstval(*values, sentinel=None):
     raise ValueError(f"All values are {sentinel}")
 
 
-def zipsort(a: Sequence[T], b: Sequence[T2], key: Callable = None, reverse=False
+def zipsort(a: Sequence[T], b: Sequence[T2], key: Callable | None = None, reverse=False
             ) -> tuple[list[T], list[T2]]:
     """
     Sort a and keep b in sync
@@ -387,6 +384,7 @@ def duplicates(seq: Sequence[T], mincount=2) -> list[T]:
     Find all elements in seq which are present at least `mincount` times
     """
     if mincount == 2:
+        from . import iterlib
         return list(iterlib.duplicates(seq))
 
     from collections import Counter
@@ -408,7 +406,7 @@ def remove_duplicates(seq: Sequence[T]) -> list[T]:
 
     .. note:: list(set(...)) does not keep order
     """
-    # In python >= 3.7 we use the fact that dicts keep order:
+    # we use the fact that dicts keep order:
     return list(dict.fromkeys(seq))
 
 
@@ -491,11 +489,7 @@ def sec2str(seconds:float, msdigits=3) -> str:
     sfrac = round((s - sint), msdigits)
     fmt = f"%.{msdigits}g"
     msstr = (fmt % sfrac)[1:2+msdigits]
-
-    if h > 0:
-        return f"{h}:{m:02}:{sint:02}{msstr}"
-    else:
-        return f"{m}:{sint:02}{msstr}"
+    return f"{h}:{m:02}:{sint:02}{msstr}" if h > 0 else f"{m}:{sint:02}{msstr}"
 
 
 def parse_time(t: str) -> float:
@@ -524,19 +518,6 @@ def parse_time(t: str) -> float:
         raise ValueError("Format not understood")
 
 
-def sumlist(seq: Iterable[list[T]]) -> list[T]:
-    """
-    Concatenate multiple lists to one big list
-
-    Args:
-        seq: a list or iterable of lists
-
-    Returns:
-        the concatenated list
-    """
-    return sum(seq, [])
-
-
 # ------------------------------------------------------------
 #
 #     namedtuple utilities
@@ -557,11 +538,12 @@ def namedtuple_addcolumn(namedtuples, seq, column_name: str, classname=""):  # t
     Returns:
         a list of namedtuples with the added column
     """
+    from collections import namedtuple
     t0 = namedtuples[0]
     assert isinstance(t0, tuple) and hasattr(t0, "_fields"), "namedtuples should be a seq. of namedtuples"
     name = classname or namedtuples[0].__class__.__name__ + '_' + column_name  # type: str
-    New_Tup = _namedtuple(name, t0._fields + (column_name,))  # type: ignore
-    newtuples = [New_Tup(*(t + (value,)))
+    NewTup = namedtuple(name, t0._fields + (column_name,))  # type: ignore
+    newtuples = [NewTup(*(t + (value,)))
                  for t, value in zip(namedtuples, seq)]
     return newtuples
 
@@ -591,8 +573,9 @@ def namedtuples_renamecolumn(namedtuples: list, oldname: str, newname: str, clas
         classname = "%s_R" % namedtuples[0].__class__.__name__
     newfields = [field if field != oldname else newname
                  for field in namedtuples[0]._fields]
-    New_Tup = _namedtuple(classname, newfields)
-    newtuples = [New_Tup(*t) for t in namedtuples]
+    from collections import namedtuple
+    NewTup = namedtuple(classname, newfields)
+    newtuples = [NewTup(*t) for t in namedtuples]
     return newtuples
 
 
@@ -625,10 +608,12 @@ def namedtuple_extend(name: str, orig, columns: str | Sequence[str]):
         Vec3(x=10, y=20, z=30)
 
     """
+    from collections import namedtuple
+
     if isinstance(columns, str):
         columns = columns.split()
     fields = orig._fields + tuple(columns)
-    N = _namedtuple(name, fields)
+    N = namedtuple(name, fields)
 
     def new_from_orig(orig, *args, **kws):
         """
@@ -688,6 +673,7 @@ def asnumber(obj, accept_fractions=True, accept_expon=False
         return obj
     elif isinstance(obj, str):
         if accept_fractions and "/" in obj:
+            from fractions import Fraction
             return Fraction(obj)
         try:
             asint = int(obj)
@@ -800,14 +786,10 @@ def moses(pred: Callable[[T], bool], seq: Iterable[T]
         >>> moses(lambda x:x > 5, range(10))
         ([6, 7, 8, 9], [0, 1, 2, 3, 4, 5])
     """
-    trueitems = []
-    falseitems = []
+    trueseq, falseseq = [], []
     for x in seq:
-        if pred(x):
-            trueitems.append(x)
-        else:
-            falseitems.append(x)
-    return trueitems, falseitems
+        (trueseq if pred(x) else falseseq).append(x)
+    return trueseq, falseseq
 
 
 def allequal(xs: Sequence) -> bool:
@@ -845,8 +827,8 @@ def can_be_pickled(obj) -> bool:
     return obj == obj2
 
 
-def snap_to_grid(x: number_t, tick: number_t, offset: number_t=0, nearest=True
-                 ) -> number_t:
+def snap_to_grid(x: num_t, tick: num_t, offset: num_t = 0, nearest=True
+                 ) -> num_t:
     """
     Find the nearest slot in a grid
 
@@ -923,7 +905,7 @@ def snap_array(X: np.ndarray,
 def _snap_array_nearest(X: np.ndarray,
                         tick: number_t,
                         offset: number_t = 0.,
-                        out: Optional[np.ndarray] = None
+                        out: np.ndarray | None = None
                         ) -> np.ndarray:
     if out is None:
         out = X.copy()
@@ -957,70 +939,7 @@ def _snap_array_floor(X: np.ndarray, tick:float, offset=0., out: np.ndarray=None
     return arr
 
 
-def snap_to_grids(x: number_t, ticks: Sequence[number_t],
-                  offsets: Sequence[number_t] = None, mode='nearest'
-                  ) -> number_t:
-    """
-    Snap x to the nearest slot within multiple overlapping grids
-
-    A regular grid is defined as ``x = offset + tick*n``, where ``n`` is an integer
-    from -inf to inf.
-
-    Args:
-        x: a number or a seq. of numbers
-        ticks: a seq. of ticks, each tick defines a grid
-        offsets: a seq. of offsets, or None to set offset to 0 for each grid
-        mode: one of 'floor', 'ceil', 'nearest'
-
-    Returns:
-        the snapped value
-
-    Given a list of regular grids, snap the value of x to this grid
-
-    Example
-    ~~~~~~~
-
-    snap a time to a grid of 16th notes
-    >>> snap_to_grids(0.3, [1/4])
-    0.25
-
-    snap a time to a multiple grid of 16ths, 8th note triplets, etc.
-    >>> snap_to_grids(0.3, [1/8, 1/6, 1/5])
-    0.3333333333333333
-
-    """
-    assert isinstance(ticks, (list, tuple))
-    assert offsets is None or isinstance(offsets, (list, tuple))
-
-    if offsets is None:
-        offsets = [Fraction(0)] * len(ticks)
-
-    if any(tick <= 0 for tick in ticks):
-        raise ValueError(f"all ticks must be > 0, got {ticks}")
-
-    def snap_round(x, tick, offset):
-        return round((x - offset) * (1 / tick)) * tick + offset
-
-    def snap_floor(x, tick, offset):
-        return math.floor((x - offset) * (1 / tick)) * tick + offset
-
-    def snap_ceil(x, tick, offset):
-        return math.ceil((x - offset) * (1 / tick)) * tick + offset
-
-    func = {
-        'floor': snap_floor,
-        'ceil': snap_ceil,
-        'nearest': snap_round,
-        'round': snap_round
-    }.get(mode)
-    if func is None:
-        raise ValueError(f"mode should be one of 'floor', 'ceil', 'round', but got {mode}")
-    quants = [func(x, t, o) for t, o in zip(ticks, offsets)]
-    quants.sort(key=lambda quant: abs(quant - x))
-    return quants[0]
-
-
-def distribute_in_zones(x: number_t, split_points: Sequence[number_t], side="left") -> int:
+def distribute_in_zones(x: num_t, split_points: Sequence[num_t], side="left") -> int:
     """
     Returns the index of a "zone" where to place x.
 
@@ -1061,7 +980,7 @@ def distribute_in_zones(x: number_t, split_points: Sequence[number_t], side="lef
     return imin
 
 
-def _distribute_in_zones_right(x: number_t, split_points: Sequence[number_t]) -> int:
+def _distribute_in_zones_right(x: num_t, split_points: Sequence[num_t]) -> int:
     """
     the same as distribute_in_zones, but with right inclusive zones
     """
@@ -1076,7 +995,7 @@ def _distribute_in_zones_right(x: number_t, split_points: Sequence[number_t]) ->
     return imin
 
 
-def seq_contains(seq, subseq) -> Optional[tuple[int, int]]:
+def seq_contains(seq: Sequence[T], subseq: Sequence[T]) -> tuple[int, int] | None:
     """
     Returns the (start, end) indexes if seq contains subseq, or None
 
@@ -1095,36 +1014,7 @@ def seq_contains(seq, subseq) -> Optional[tuple[int, int]]:
     return None
 
 
-def pick_regularly(seq: Sequence[T] | np.ndarray, numitems: int, start_idx=0,
-                   end_idx=0
-                   ) -> Union[np.ndarray, list[T]]:
-    """
-    Given a sequence, pick `numitems` from it at regular intervals
-
-    The first and the last items are always included. The behaviour
-    is similar to numpy's linspace
-
-    Args:
-        seq: a sequence of items (a list of items or a numpy array)
-        numitems: the number of items to pick from seq
-        start_idx: if given, the index to start picking from
-        end_idx: if given, the index to stop
-
-    Returns:
-        a list of the picked items (as a list if the input was a list, or a numpy
-        array if the input was an array)
-    """
-    if end_idx == 0:
-        end_idx = len(seq) - 1
-    indexes = np.linspace(start_idx, end_idx, numitems)
-    np.round(indexes, out=indexes)
-    if isinstance(seq, np.ndarray):
-        return seq[indexes]
-    else:
-        return [seq[i] for i in indexes]
-
-
-def deepupdate(orig, updatewith):
+def deepupdate(orig: dict, updatewith: dict) -> dict:
     """
     recursively update orig with updatewith
     """
@@ -1144,26 +1034,6 @@ def deepupdate(orig, updatewith):
 # ------------------------------------------------------------
 
 
-def fig2data(fig) -> np.ndarray:
-    """
-    Convert a Matplotlib figure to a 4D numpy array with RGBA channels
-
-    Args:
-        fig: a matplotlib figure
-
-    Returns:
-        a numpy 3D array of RGBA values
-    """
-    fig.canvas.draw()        # draw the renderer
-    # Get the RGBA buffer from the figure
-    w, h = fig.canvas.get_width_height()
-    buf = np.fromstring(fig.canvas.tostring_argb(), dtype=np.uint8)
-    buf.shape = (w, h, 4)
-    # canvas.tostring_argb give pixmap in ARGB mode.
-    # Roll the ALPHA channel to have it in RGBA mode
-    buf = np.roll(buf, 3, axis=2)
-    return buf
-
 
 def pixels_to_cm(pixels: int, dpi=300) -> float:
     """
@@ -1256,63 +1126,7 @@ def page_dinsize_to_mm(pagesize: str, pagelayout: str) -> tuple[float, float]:
 #    Decorators
 #
 # ------------------------------------------------------------
-
-
-def returns_tuple(names, recname=None):
-    """
-    Decorator - Makes the function return a namedtuple with the given names.
-
-    Args:
-        names: as passed to namedtuple, either a space-divided string,
-            or a sequence of strings
-        recname: a name to be given to the result as a whole. If nothing is
-            given, the name of the decorated function is used.
-
-    Example::
-
-        >>> @returns_tuple("minval maxval")
-        ... def minmax(seq):
-        ...     return min(seq), max(seq)
-        >>> result = minmax([1, 2, 3, 4])
-        >>> result.minval
-        1
-        >>> result.maxval
-        4
-
-    """
-    from decorator import decorator
-    from collections import defaultdict
-    registry = {}
-    if recname:
-        nt = _namedtuple(recname, names)
-        registry = defaultdict(lambda: nt)
-    else:
-        registry = {}
-
-    @decorator
-    def wrapper(func, *args, **kws):
-        result = func(*args, **kws)
-        nt = registry.get(func.__name__)
-        if nt is None:
-            nt = _namedtuple(func.__name__, names)
-            registry[func.__name__] = nt
-        return nt(*result)
-    return wrapper
-
-
-def returns_tuples(names, recname):
-    """
-    Decorator to make a function return a RecordList instead of a list of tuples
-    """
-    from decorator import decorator
-    from .containers import RecordList
-
-    @decorator
-    def wrapper(func, *args, **kws):
-        result = func(*args, **kws)
-        return RecordList(result, recname)
-    return wrapper
-
+#
 
 def public(f):
     """
@@ -1332,7 +1146,7 @@ def public(f):
     return f
 
 
-def singleton(cls):
+def singleton(cls: type):
     """
     A class decorator to create a singleton class
 
@@ -1361,68 +1175,6 @@ def singleton(cls):
 
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-def istype(obj, *types) -> bool:
-    """
-    Examples::
-
-        >>> istype(1, int, float)
-        True
-        >>> istype((4, 0.5), (int, float))
-        True
-        >>> istype([4, 3], [int])
-        True
-        >>> istype({"foo": 4, "bar": 3}, {str:int})
-        True
-    """
-    if len(types) > 1:
-        return isinstance(obj, types)
-
-    t = types[0]
-
-    if isinstance(t, type):
-        return isinstance(obj, t)
-
-    if isinstance(t, tuple):
-        return (isinstance(obj, tuple) and
-                len(obj) == len(t) and
-                all(istype(subobj, subt) for subobj, subt in zip(obj, t)))
-    elif isinstance(t, list):
-        if len(t) == 0:
-            return isinstance(obj, list)
-        elif len(t) == 1:
-            subt = t[0]
-            return isinstance(obj, list) and all(istype(subobj, subt)
-                                                 for subobj in obj)
-        else:
-            raise ValueError(f"T should be [type], but found: {t}")
-    elif isinstance(t, dict):
-        assert len(t) == 1
-        keyt, valt = list(t.items())[0]
-        return all(istype(key, keyt) and istype(value, valt)
-                   for key, value in obj.items())
-    else:
-        raise TypeError("T type not supported, see examples")
-
-
-def assert_type(x, *types) -> bool:
-    """
-    Returns True if x is of the given type, raises TypeError otherwise
-
-    See istype for examples
-
-    Can be used as::
-
-        assert_type(x, int, float)
-
-    or::
-
-        assert assert_type(x, [int])
-    """
-    if not istype(x, *types):
-        raise TypeError(type_error_msg(x, *types))
-    return True
-
-
 def type_error_msg(x, *expected_types):
     """
     To be used when raising a TypeError
@@ -1443,8 +1195,8 @@ def type_error_msg(x, *expected_types):
 
 # --- crossplatform ---
 
-def _open_with_standard_app(path: str, wait:Union[str, bool]=False, min_wait=0.5,
-                            timeout:float=None
+def _open_with_standard_app(path: str, wait: str | bool = False, min_wait=0.5,
+                            timeout=0., capture_stdout=True, capture_stderr=True
                             ) -> None:
     """
     Open path with the app defined to handle it at the os level
@@ -1463,30 +1215,36 @@ def _open_with_standard_app(path: str, wait:Union[str, bool]=False, min_wait=0.5
         timeout: a timeout for waiting on modified
 
     """
-    import subprocess, time
-    platform = _sys.platform
+    import subprocess
+    import time
     proc = None
-    if platform == 'linux':
-        proc = subprocess.Popen(["xdg-open", path])
-    elif platform == "win32":
+    if _sys.platform == 'linux':
+        proc = subprocess.Popen(
+            ["xdg-open", path],
+            stdout=subprocess.PIPE if capture_stdout else None,
+            stderr=subprocess.PIPE if capture_stderr else None)
+    elif _sys.platform == "win32":
         # this function exists only in windows
-        _os.startfile(path)
-    elif platform == "darwin":
-        proc = subprocess.Popen(["open", path])
+        _os.startfile(path)  # type: ignore
+    elif _sys.platform == "darwin":
+        proc = subprocess.Popen(
+            ["open", path],
+            stdout=subprocess.PIPE if capture_stdout else None,
+            stderr=subprocess.PIPE if capture_stderr else None)
         min_wait = max(min_wait, 1)
     else:
-        raise RuntimeError(f"platform {platform} not supported")
+        raise RuntimeError(f"platform {_sys.platform} not supported")
 
-    t0 = time.time()
     if wait == "modified":
         wait_for_file_modified(path, timeout=timeout or 36000)
     elif wait:
         from emlib import dialogs
-        if platform == "win32":
+        if _sys.platform == "win32":
             dialogs.showInfo("Close this dialog when finished")
-        else:
+        elif proc is not None:
+            t0 = time.time()
             proc.wait()
-            if time.time()-t0 < min_wait:
+            if time.time() - t0 < min_wait:
                 dialogs.showInfo("Close this dialog when finished")
 
 
@@ -1501,9 +1259,11 @@ def open_with_app(path: str,
                   wait: bool | str = False,
                   shell=False,
                   min_wait=0.5,
-                  timeout=None) -> None:
+                  timeout=0.,
+                  capture_stdout=True,
+                  capture_stderr=True) -> subprocess.Popen | None:
     """
-    Open a given file with a given app.
+    Open a document with a given app.
 
     It can either wait on the app to exit or wait until the file
     was modified. The app can be either a command as a string or a
@@ -1527,7 +1287,8 @@ def open_with_app(path: str,
     """
     if not app:
         assert not shell
-        _open_with_standard_app(path, wait=wait, min_wait=min_wait, timeout=timeout)
+        _open_with_standard_app(path, wait=wait, min_wait=min_wait, timeout=timeout,
+                                capture_stderr=capture_stderr, capture_stdout=capture_stdout)
         return
 
     import subprocess
@@ -1555,7 +1316,7 @@ def open_with_app(path: str,
             dialogs.showInfo("Close this dialog when finished")
 
 
-def wait_for_file_modified(path:str, timeout:float=None) -> bool:
+def wait_for_file_modified(path: str, timeout: int | float = 0.) -> bool:
     """
     Wait until file is modified.
 
@@ -1565,7 +1326,7 @@ def wait_for_file_modified(path:str, timeout:float=None) -> bool:
 
     Args:
         path: the path of the file to monitor
-        timeout: how long should we wait for
+        timeout: how long should we wait for, in seconds
 
     Returns:
         True if the file was modified, False if it wasn't or if the operation
@@ -1576,8 +1337,7 @@ def wait_for_file_modified(path:str, timeout:float=None) -> bool:
     directory, base = _os.path.split(path)
     if not directory:
         directory = "."
-    handler = PatternMatchingEventHandler([base], ignore_patterns="",
-                                          ignore_directories=True, case_sensitive=True)
+    handler = PatternMatchingEventHandler(patterns=[base], ignore_directories=True, case_sensitive=True)
     observer = Observer()
     modified = False
 
@@ -1595,7 +1355,7 @@ def wait_for_file_modified(path:str, timeout:float=None) -> bool:
     return modified
 
 
-def first_existing_path(*paths: str, default="~") -> str:
+def first_existing_path(*paths: str) -> str | None:
     """
     Returns the first path in paths which exists
 
@@ -1605,20 +1365,21 @@ def first_existing_path(*paths: str, default="~") -> str:
             not checked that this default path exists.
 
     Returns:
-        the first existing path within the values given
+        the first existing path within the values given, None if no
+        match was found
 
     """
     for p in paths:
         p = _os.path.expanduser(p)
         if _os.path.exists(p):
             return p
-    return _os.path.expanduser(default)
+    return None
 
 
 def html_table(rows: list,
                headers: list[str],
-               maxwidths: Optional[list[int]] = None,
-               rowstyles: Optional[list[str]] = None,
+               maxwidths: list[int] | None = None,
+               rowstyles: list[str] | None = None,
                tablestyle='',
                headerstyle=''
                ) -> str:
@@ -1650,14 +1411,14 @@ def html_table(rows: list,
     if maxwidths is None:
         maxwidths = [0] * len(headers)
     if rowstyles is None:
-        rowstyles = [None] * len(headers)
+        rowstyles = [''] * len(headers)
     for colname in headers:
         _(f'<th style="text-align:left">{colname}</th>')
     _("</tr></thead><tbody>")
     for row in rows:
         _("<tr>")
         for cell, maxwidth, rowstyle in zip(row, maxwidths, rowstyles):
-            if rowstyle is not None:
+            if rowstyle:
                 cell = f'<span style="{rowstyle}">{cell}</span>'
             if maxwidth > 0:
                 _(f'<td style="text-align:left;max-width:{maxwidth}px;">{cell}</td>')
@@ -1668,7 +1429,7 @@ def html_table(rows: list,
     return "".join(parts)
 
 
-def print_table(rows:list, headers=(), tablefmt:str='', showindex=True, floatfmt: str|tuple[str, ...]='') -> None:
+def print_table(rows: list, headers=(), tablefmt='', showindex=True, floatfmt: str|tuple[str, ...]='g') -> None:
     """
     Print rows as table
 
@@ -1685,6 +1446,7 @@ def print_table(rows:list, headers=(), tablefmt:str='', showindex=True, floatfmt
     if not rows:
         raise ValueError("rows is empty")
 
+    import dataclasses
     row0 = rows[0]
     if dataclasses.is_dataclass(row0):
         if not headers:
@@ -1700,6 +1462,7 @@ def print_table(rows:list, headers=(), tablefmt:str='', showindex=True, floatfmt
                         f", got {type(row0)}")
 
     import tabulate
+    from .envir import inside_jupyter
     if inside_jupyter():
         from IPython.display import HTML, display
         if not tablefmt:
@@ -1707,23 +1470,23 @@ def print_table(rows:list, headers=(), tablefmt:str='', showindex=True, floatfmt
         disable_numparse = not floatfmt
         s = tabulate.tabulate(rows, headers=headers, disable_numparse=disable_numparse,
                               tablefmt=tablefmt, showindex=showindex, stralign='left',
-                              floatfmt=floatfmt if floatfmt else tabulate._DEFAULT_FLOATFMT)
+                              floatfmt=floatfmt)
         if tablefmt == 'html':
             display(HTML(s))
         else:
             print(s)
     else:
         print(tabulate.tabulate(rows, headers=headers, showindex=showindex, tablefmt=tablefmt,
-                                floatfmt=floatfmt if floatfmt else tabulate._DEFAULT_FLOATFMT))
+                                floatfmt=floatfmt))
 
 
-def replace_sigint_handler(handler):
+def replace_sigint_handler(handler: Callable[[None], None]):
     """
     Replace current SIGINT hanler with the given one, return the old one
 
     Args:
         handler: the new handler. A handler is a function taking no
-        parameters and returning nothing
+            parameters and returning nothing
 
     Returns:
         the old handler
@@ -1762,7 +1525,7 @@ class temporary_sigint_handler:
 
 def simplify_breakpoints(bps: list[T],
                          coordsfunc: Callable,
-                         tolerance: number_t = 0.01
+                         tolerance= 0.01
                          ) -> list[T]:
     """
     Simplify breakpoints in a breakpoint function
