@@ -4,11 +4,12 @@ Miscellaneous utilities for working with numpy arrays
 from __future__ import annotations
 import numpy as np
 from numpy.lib.stride_tricks import as_strided
+from typing import Iterator
 
 
 def interlace(*arrays: np.ndarray) -> np.ndarray:
     """
-    Interweave multiple arrays into a flat array in the form
+    Interleave multiple arrays into a flat array in the form
 
     Example::
 
@@ -26,9 +27,9 @@ def interlace(*arrays: np.ndarray) -> np.ndarray:
         a 1D array with the elements of the given arrays interleaved
 
     """
-    assert all(a.size == arrays[0].size and a.dtype == arrays[0].dtype for a in arrays)
-    size = arrays[0].size * len(arrays)
-    out = np.empty((size,), dtype=arrays[0].dtype)
+    size = arrays[0].size
+    assert all(array.size == size and array.dtype == arrays[0].dtype for array in arrays)
+    out = np.empty((size * len(arrays),), dtype=arrays[0].dtype)
     for i, a in enumerate(arrays):
         out[i::len(arrays)] = a
     return out
@@ -55,7 +56,7 @@ def npunzip(a: np.ndarray) -> np.ndarray:
     return a.transpose()
 
 
-def zipsort(a: np.ndarray, b: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+def zipsort(a: np.ndarray, b: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
     """
     Sort one array, keep the other synched
 
@@ -68,7 +69,7 @@ def zipsort(a: np.ndarray, b: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
     indices = a.argsort()
     return a[indices], b[indices]
 
-    
+
 def smooth(a: np.ndarray, kind="running", strength=0.05) -> np.ndarray:
     """
     Smooth the values in a
@@ -84,7 +85,7 @@ def smooth(a: np.ndarray, kind="running", strength=0.05) -> np.ndarray:
     """
     assert len(a) > 3
     if kind == "running":
-        N = len(a) if strength is None else min(len(a) * 0.5 * strength, 3)
+        N = len(a) if strength is None else min(round(len(a) * 0.5 * strength), 3)
         K = np.ones(N, dtype=float) / N
         a_smooth = np.convolve(a, K, mode='same')
     else:
@@ -92,7 +93,8 @@ def smooth(a: np.ndarray, kind="running", strength=0.05) -> np.ndarray:
     return a_smooth
 
 
-def overlapping_frames(y: np.ndarray, frame_length: int, hop_length: int):
+def overlapping_frames(y: np.ndarray, frame_length: int, hop_length: int
+                       ) -> np.ndarray:
     """
     Slice a time series into overlapping frames.
 
@@ -143,16 +145,20 @@ def overlapping_frames(y: np.ndarray, frame_length: int, hop_length: int):
     return y_frames
 
 
-def chunks(data: np.ndarray, chunksize: int, hop:int=None, padwith=0):
+def chunks(data: np.ndarray,
+           chunksize: int,
+           hop: int | None = None,
+           padwith: float | int | None = None
+           ) -> Iterator[np.ndarray]:
     """
     Iterate over data in chunks of chunksize. Returns a generator
 
     Args:
         data: the array to be iterated in chunks
         chunksize: the size of each chunk
-        hop: the amount of elements to skip between chunks
-        padwidth: value to pad when a chunk is not big enough
-            Give None to avoid padding
+        hop: the amount of elements to skip between chunks, None to use chunksize as hop size
+        padwith: value to pad when a chunk is not big enough, None to avoid padding. The type
+            depends on the data type of the given array
 
     Returns:
         a generator with chunks of data of chunksize or less
@@ -178,9 +184,9 @@ def chunks(data: np.ndarray, chunksize: int, hop:int=None, padwith=0):
             n += hop
 
 
-def padarray(arr, numelements, padwith=0):
+def padarray(arr: np.ndarray, numelements: int, padwith=0) -> np.ndarray:
     """
-    Pad a 1D array to the right with 0s, or a 2D array down with zeros
+    Pad a 1D array to the right, or a 2D array down
 
     Pad 1D with 2 elements::
 
@@ -203,7 +209,7 @@ def padarray(arr, numelements, padwith=0):
         raise ValueError("Only 1D or 2D arrays supported")
 
 
-def linlin(xs:np.ndarray, x0:float, x1:float, y0:float, y1: float) -> np.ndarray:
+def linlin(xs: np.ndarray, x0: float, x1: float, y0: float, y1: float) -> np.ndarray:
     """
     Map xs from range x0-x1 to y0-y1
 
@@ -225,10 +231,10 @@ def linlin(xs:np.ndarray, x0:float, x1:float, y0:float, y1: float) -> np.ndarray
     return xs
 
 
-def astype(a: np.ndarray, typedescr):
+def astype(a: np.ndarray, typedescr) -> np.ndarray:
     """
     The same as: `if a.dtype != typedescr: a = as.astype(typedescr)`
-    
+
     """
     return a if a.dtype == typedescr else a.astype(typedescr)
 
@@ -283,3 +289,29 @@ def nearestindex(a: np.ndarray, grid: np.ndarray, left=True, right=True
     else:
         raise ValueError("At least left or right must be true")
 
+
+def arange_numvalues(step: float, numvalues: int, start=0.
+                     ) -> np.ndarray:
+    """
+    Construct an array of `numvalues` starting from `start` with `step`
+
+    Args:
+        step: the step between each value
+        numvalues: the number of values of the array
+        start: the starting value
+
+    Returns:
+        the new array
+
+    Example
+    -------
+
+    >>> arange_numvalues(0.1, 10)
+    array([0. , 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9])
+
+    >>> arange_numvalues(0.1, 20, 10)
+    array([10.  , 10.01, 10.02, 10.03, 10.04, 10.05, 10.06, 10.07, 10.08,
+           10.09, 10.1 , 10.11, 10.12, 10.13, 10.14, 10.15, 10.16, 10.17,
+           10.18, 10.19])
+    """
+    return np.arange(start, step * numvalues + start, step)
