@@ -2,7 +2,6 @@
 Routines for working with text
 """
 from __future__ import annotations
-import sys
 import textwrap
 import re
 
@@ -11,19 +10,33 @@ if TYPE_CHECKING:
     from typing import Sequence, Callable
 
 
-def stripLines(text: str) -> str:
+def stripLines(text: str, which='all', splitregex=''):
     """
-    Like ``str.strip`` but operates on lines as a whole.
+    Strip leading and trailing empty lines
 
-    Removes empty lines at the beginning or end of text,
-    without touching lines in between.
+    Args:
+        text: the text to work on
+        which: one of 'all', 'top', 'bottom', where all removes all
+            empty lines at the top and bottom, top removes only
+            leading empty lines and bottom removes only
+            trailing empty lines
+
+    Returns:
+        the transformed text
     """
-    lines = text.striplines()
-    lines = stripLinesTopAndBottom(lines)
-    return "\n".join(lines)
- 
+    lines = text.splitlines() if not splitregex else re.split(text, splitregex)
+    if which == 'all':
+        lines = linesStrip(lines)
+    elif which == 'top':
+        lines = linesStripTop(lines)
+    elif which == 'bottom':
+        lines = linesStripBottom(lines)
+    else:
+        raise ValueError(f"Expected one of 'all', 'top', 'bottom', got '{which}'")
+    return '\n'.join(lines)
 
-def stripLinesTopAndBottom(lines: list[str]) -> list[str]:
+
+def linesStrip(lines: list[str]) -> list[str]:
     """
     Remove empty lines from the top and bottom
 
@@ -43,41 +56,17 @@ def stripLinesTopAndBottom(lines: list[str]) -> list[str]:
     return lines[startidx:len(lines)-endidx]
 
 
-def splitAndStripLines(text: str | list[str], regexp: str = None) -> list[str]:
-    """
-    Splits `text` into lines and removes empty lines at the beginning and end
-
-    Returns the split lines in between
-
-    Args:
-        text: the text to split
-        regexp: if given, use this regexp to split the lines. Otherwise uses ``str.splitlines``
-
-    Returns:
-        the list of lines
-    """
-    if isinstance(text, list):
-        lines = text
-    else:
-        lines = re.split(regexp, text) if regexp else text.splitlines()
-    return stripLinesTopAndBottom(lines)
-
-
-def reindent(text: str, prefix="", stripEmptyLines=True) -> str:
+def reindent(text: str, prefix: str) -> str:
     """
     Reindent a given text. Replaces the indentation with a new prefix.
 
     Args:
         text: the text to reindent
         prefix: the new prefix to add to each line
-        stripEmptyLines: if True, remove any empty lines at the beginning
-            or end of ``text``
 
     Returns:
         the reindented text
     """
-    if stripEmptyLines:
-        text = stripLines(text)
     text = textwrap.dedent(text)
     if prefix:
         text = textwrap.indent(text, prefix=prefix)
@@ -119,7 +108,7 @@ def matchIndentation(code: str, modelcode: str) -> str:
     return textwrap.indent(code, prefix=" " * indentation)
 
 
-def stripLinesTop(lines: list[str]) -> list[str]:
+def linesStripTop(lines: list[str]) -> list[str]:
     """
     Remove empty lines from the top
 
@@ -137,7 +126,7 @@ def stripLinesTop(lines: list[str]) -> list[str]:
     return lines[i:]
 
 
-def stripLinesBottom(lines: list[str], maxlines: int = 0) -> list[str]:
+def linesStripBottom(lines: list[str], maxlines: int = 0) -> list[str]:
     """
     Strip empty lines from the end of the list
 
@@ -158,7 +147,7 @@ def stripLinesBottom(lines: list[str], maxlines: int = 0) -> list[str]:
     return lines
 
 
-def joinPreservingIndentation(fragments: Sequence[str], maxEmptyLines: int = None) -> str:
+def joinPreservingIndentation(fragments: Sequence[str]) -> str:
     """
     Like join, but preserving indentation
 
@@ -173,15 +162,6 @@ def joinPreservingIndentation(fragments: Sequence[str], maxEmptyLines: int = Non
     if any(not isinstance(fragment, str) for fragment in fragments):
         fragment = next(_ for _ in fragments if not isinstance(_, str))
         raise TypeError(f"Expected a string, got {fragment}")
-    if maxEmptyLines is not None:
-        if maxEmptyLines == 0:
-            fragments = [stripLines(frag) for frag in fragments]
-        else:
-            splitfragments = [fragment.splitlines() for fragment in fragments]
-            splitfragments = [stripLinesTop(frag) for frag in splitfragments]
-            splitfragments = [stripLinesBottom(frag, maxlines=maxEmptyLines)
-                              for frag in splitfragments]
-            fragments = ["\n".join(frag) for frag in splitfragments]
     jointtext = "\n".join(textwrap.dedent(frag) for frag in fragments if frag)
     numspaces = getIndentation(fragments[0])
     if numspaces:
@@ -371,7 +351,6 @@ _fractions = {
     (5, 6): "⅚",
     (1, 7): "⅐",
     (1, 8): "⅛",
-    (2, 4): "¼",
     (3, 8): "⅜",
     (4, 8): "½",
     (5, 8): "⅝",
